@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2011 Sebastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2012 Sebastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -20,6 +20,8 @@
 #ifndef __WEECHAT_HOOK_H
 #define __WEECHAT_HOOK_H 1
 
+#include <unistd.h>
+
 #ifdef HAVE_GNUTLS
 #include <gnutls/gnutls.h>
 #endif
@@ -106,7 +108,7 @@ struct t_hook
     int running;                       /* 1 if hook is currently running    */
     int priority;                      /* priority (to sort hooks)          */
     void *callback_data;               /* data sent to callback             */
-    
+
     /* hook data (depends on hook type) */
     void *hook_data;                   /* hook specific data                */
     struct t_hook *prev_hook;          /* link to previous hook             */
@@ -126,17 +128,17 @@ struct t_hook_command
     char *args;                         /* (for /help) command arguments    */
     char *args_description;             /* (for /help) args long description*/
     char *completion;                   /* template for completion          */
-    
+
     /* templates */
     int cplt_num_templates;             /* number of templates for compl.   */
     char **cplt_templates;              /* completion templates             */
     char **cplt_templates_static;       /* static part of template (at      */
                                         /* beginning                        */
-    
+
     /* arguments for each template */
     int *cplt_template_num_args;        /* number of arguments for template */
     char ***cplt_template_args;         /* arguments for each template      */
-    
+
     /* concatenation of arg N for each template */
     int cplt_template_num_args_concat; /* number of concatened arguments    */
     char **cplt_template_args_concat;  /* concatened arguments              */
@@ -178,7 +180,7 @@ struct t_hook_fd
     t_hook_callback_fd *callback;      /* fd callback                       */
     int fd;                            /* socket or file descriptor         */
     int flags;                         /* fd flags (read,write,..)          */
-    int error;                         /* contains errno if error occured   */
+    int error;                         /* contains errno if error occurred  */
                                        /* with fd                           */
 };
 
@@ -192,6 +194,7 @@ struct t_hook_process
 {
     t_hook_callback_process *callback; /* process callback (after child end)*/
     char *command;                     /* command executed by child         */
+    struct t_hashtable *options;       /* options for process (see doc)     */
     long timeout;                      /* timeout (ms) (0 = no timeout)     */
     int child_read[2];                 /* to read data in pipe from child   */
     int child_write[2];                /* to write data in pipe for child   */
@@ -396,6 +399,7 @@ struct t_hook_focus
 
 /* hook variables */
 
+extern char *hook_type_string[];
 extern struct t_hook *weechat_hooks[];
 extern struct t_hook *last_weechat_hook[];
 
@@ -440,6 +444,12 @@ extern struct t_hook *hook_process (struct t_weechat_plugin *plugin,
                                     int timeout,
                                     t_hook_callback_process *callback,
                                     void *callback_data);
+extern struct t_hook *hook_process_hashtable (struct t_weechat_plugin *plugin,
+                                              const char *command,
+                                              struct t_hashtable *options,
+                                              int timeout,
+                                              t_hook_callback_process *callback,
+                                              void *callback_data);
 extern struct t_hook *hook_connect (struct t_weechat_plugin *plugin,
                                     const char *proxy, const char *address,
                                     int port, int sock, int ipv6,
