@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2011 Sebastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2012 Sebastien Helleu <flashcode@flashtux.org>
  * Copyright (C) 2006 Emmanuel Bouthenot <kolter@openics.org>
  *
  * This file is part of WeeChat, the extensible chat client.
@@ -137,7 +137,7 @@ void
 utf8_normalize (char *string, char replacement)
 {
     char *error;
-    
+
     while (string && string[0])
     {
         if (utf8_is_valid (string, &error))
@@ -156,9 +156,9 @@ utf8_prev_char (const char *string_start, const char *string)
 {
     if (!string || (string <= string_start))
         return NULL;
-    
+
     string--;
-    
+
     if (((unsigned char)(string[0]) & 0xC0) == 0x80)
     {
         /* UTF-8, at least 2 bytes */
@@ -197,7 +197,7 @@ utf8_next_char (const char *string)
 {
     if (!string)
         return NULL;
-    
+
     /* UTF-8, 2 bytes: 110vvvvv 10vvvvvv */
     if (((unsigned char)(string[0]) & 0xE0) == 0xC0)
     {
@@ -237,12 +237,12 @@ int
 utf8_char_int (const char *string)
 {
     const unsigned char *ptr_string;
-    
+
     if (!string)
         return 0;
-    
+
     ptr_string = (unsigned char *)string;
-    
+
     /* UTF-8, 2 bytes: 110vvvvv 10vvvvvv */
     if ((ptr_string[0] & 0xE0) == 0xC0)
     {
@@ -293,10 +293,10 @@ utf8_wide_char (const char *string)
 {
     int char_size;
     wint_t result;
-    
+
     if (!string || !string[0])
         return WEOF;
-    
+
     char_size = utf8_char_size (string);
     switch (char_size)
     {
@@ -333,7 +333,7 @@ utf8_char_size (const char *string)
 {
     if (!string)
         return 0;
-    
+
     return utf8_next_char (string) - string;
 }
 
@@ -345,10 +345,10 @@ int
 utf8_strlen (const char *string)
 {
     int length;
-    
+
     if (!string)
         return 0;
-    
+
     length = 0;
     while (string && string[0])
     {
@@ -367,10 +367,10 @@ utf8_strnlen (const char *string, int bytes)
 {
     char *start;
     int length;
-    
+
     if (!string)
         return 0;
-    
+
     start = (char *)string;
     length = 0;
     while (string && string[0] && (string - start < bytes))
@@ -391,24 +391,24 @@ utf8_strlen_screen (const char *string)
 {
     int length, num_char;
     wchar_t *wstring;
-    
+
     if (!string)
         return 0;
-    
+
     if (!local_utf8)
         return utf8_strlen (string);
-    
+
     num_char = mbstowcs (NULL, string, 0) + 1;
     wstring = malloc ((num_char + 1) * sizeof (wstring[0]));
     if (!wstring)
         return utf8_strlen (string);
-    
+
     if (mbstowcs (wstring, string, num_char) == (size_t)(-1))
     {
         free (wstring);
         return utf8_strlen (string);
     }
-    
+
     length = wcswidth (wstring, num_char);
     free (wstring);
     return length;
@@ -422,13 +422,13 @@ int
 utf8_charcmp (const char *string1, const char *string2)
 {
     int length1, length2, i, diff;
-    
+
     if (!string1 || !string2)
         return (string1) ? 1 : ((string2) ? -1 : 0);
-    
+
     length1 = utf8_char_size (string1);
     length2 = utf8_char_size (string2);
-    
+
     i = 0;
     while ((i < length1) && (i < length2))
     {
@@ -455,18 +455,47 @@ int
 utf8_charcasecmp (const char *string1, const char *string2)
 {
     wint_t wchar1, wchar2;
-    
+
     if (!string1 || !string2)
         return (string1) ? 1 : ((string2) ? -1 : 0);
-    
+
     wchar1 = utf8_wide_char (string1);
     if ((wchar1 >= 'A') && (wchar1 <= 'Z'))
         wchar1 += ('a' - 'A');
-    
+
     wchar2 = utf8_wide_char (string2);
     if ((wchar2 >= 'A') && (wchar2 <= 'Z'))
         wchar2 += ('a' - 'A');
-    
+
+    return (wchar1 < wchar2) ? -1 : ((wchar1 == wchar2) ? 0 : 1);
+}
+
+/*
+ * utf8_charcasecmp_range: compare two utf8 chars, case is ignored
+ *                         using a range, examples:
+ *                         - range = 26: A-Z         ==> a-z
+ *                         - range = 29: A-Z [ \ ]   ==> a-z { | }
+ *                         - range = 30: A-Z [ \ ] ^ ==> a-z { | } ~
+ *                         (ranges 29 and 30 are used by some protocols like
+ *                         IRC)
+ */
+
+int
+utf8_charcasecmp_range (const char *string1, const char *string2, int range)
+{
+    wint_t wchar1, wchar2;
+
+    if (!string1 || !string2)
+        return (string1) ? 1 : ((string2) ? -1 : 0);
+
+    wchar1 = utf8_wide_char (string1);
+    if ((wchar1 >= 'A') && (wchar1 < 'A' + (unsigned int)range))
+        wchar1 += ('a' - 'A');
+
+    wchar2 = utf8_wide_char (string2);
+    if ((wchar2 >= 'A') && (wchar2 < 'A' + (unsigned int)range))
+        wchar2 += ('a' - 'A');
+
     return (wchar1 < wchar2) ? -1 : ((wchar1 == wchar2) ? 0 : 1);
 }
 
@@ -480,17 +509,17 @@ utf8_char_size_screen (const char *string)
 {
     int char_size;
     char utf_char[16];
-    
+
     if (!string)
         return 0;
-    
+
     char_size = utf8_char_size (string);
     if (char_size == 0)
         return 0;
-    
+
     memcpy (utf_char, string, char_size);
     utf_char[char_size] = '\0';
-    
+
     return utf8_strlen_screen (utf_char);
 }
 
@@ -503,7 +532,7 @@ utf8_add_offset (const char *string, int offset)
 {
     if (!string)
         return NULL;
-    
+
     while (string && string[0] && (offset > 0))
     {
         string = utf8_next_char (string);
@@ -522,10 +551,10 @@ utf8_real_pos (const char *string, int pos)
 {
     int count, real_pos;
     char *next_char;
-    
+
     if (!string)
         return pos;
-    
+
     count = 0;
     real_pos = 0;
     while (string && string[0] && (count < pos))
@@ -548,10 +577,10 @@ utf8_pos (const char *string, int real_pos)
 {
     int count;
     char *limit;
-    
+
     if (!string || !weechat_local_charset)
         return real_pos;
-    
+
     count = 0;
     limit = (char *)string + real_pos;
     while (string && string[0] && (string < limit))
@@ -570,13 +599,13 @@ char *
 utf8_strndup (const char *string, int length)
 {
     const char *end;
-    
+
     if (!string || (length < 0))
         return NULL;
-    
+
     if (length == 0)
         return strdup ("");
-    
+
     end = utf8_add_offset (string, length);
     if (!end || (end == string))
         return strdup (string);
