@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2011 Sebastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2012 Sebastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -64,19 +64,20 @@ irc_color_decode (const char *string, int keep_colors)
 {
     unsigned char *out, *ptr_string;
     int out_length, length, out_pos;
-    char str_fg[3], str_bg[3], str_color[128];
+    char str_fg[3], str_bg[3], str_color[128], str_key[128];
+    const char *remapped_color;
     int fg, bg, bold, reverse, italic, underline, rc;
-    
+
     out_length = (strlen (string) * 2) + 1;
     out = malloc (out_length);
     if (!out)
         return NULL;
-    
+
     bold = 0;
     reverse = 0;
     italic = 0;
     underline = 0;
-    
+
     ptr_string = (unsigned char *)string;
     out[0] = '\0';
     while (ptr_string && ptr_string[0])
@@ -175,11 +176,24 @@ irc_color_decode (const char *string, int keep_colors)
                                 bg %= IRC_NUM_COLORS;
                             }
                         }
-                        snprintf (str_color, sizeof (str_color),
-                                  "|%s%s%s",
-                                  (fg >= 0) ? irc_color_to_weechat[fg] : "",
-                                  (bg >= 0) ? "," : "",
-                                  (bg >= 0) ? irc_color_to_weechat[bg] : "");
+                        /* search "fg,bg" in hashtable of remapped colors */
+                        snprintf (str_key, sizeof (str_key), "%d,%d", fg, bg);
+                        remapped_color = weechat_hashtable_get (
+                            irc_config_hashtable_color_mirc_remap,
+                            str_key);
+                        if (remapped_color)
+                        {
+                            snprintf (str_color, sizeof (str_color),
+                                      "|%s", remapped_color);
+                        }
+                        else
+                        {
+                            snprintf (str_color, sizeof (str_color),
+                                      "|%s%s%s",
+                                      (fg >= 0) ? irc_color_to_weechat[fg] : "",
+                                      (bg >= 0) ? "," : "",
+                                      (bg >= 0) ? irc_color_to_weechat[bg] : "");
+                        }
                         strcat ((char *)out, weechat_color(str_color));
                     }
                     else
@@ -197,7 +211,7 @@ irc_color_decode (const char *string, int keep_colors)
                 break;
         }
     }
-    
+
     return (char *)out;
 }
 
@@ -212,12 +226,12 @@ irc_color_decode_for_user_entry (const char *string)
 {
     unsigned char *out, *ptr_string;
     int out_length, out_pos, length;
-    
+
     out_length = (strlen (string) * 2) + 1;
     out = malloc (out_length);
     if (!out)
         return NULL;
-    
+
     ptr_string = (unsigned char *)string;
     out_pos = 0;
     while (ptr_string && ptr_string[0] && (out_pos < out_length - 1))
@@ -261,9 +275,9 @@ irc_color_decode_for_user_entry (const char *string)
                 ptr_string += length;
         }
     }
-    
+
     out[out_pos] = '\0';
-    
+
     return (char *)out;
 }
 
@@ -280,12 +294,12 @@ irc_color_encode (const char *string, int keep_colors)
 {
     unsigned char *out, *ptr_string;
     int out_length, out_pos, length;
-    
+
     out_length = (strlen (string) * 2) + 1;
     out = malloc (out_length);
     if (!out)
         return NULL;
-    
+
     ptr_string = (unsigned char *)string;
     out_pos = 0;
     while (ptr_string && ptr_string[0] && (out_pos < out_length - 1))
@@ -356,9 +370,9 @@ irc_color_encode (const char *string, int keep_colors)
                 ptr_string += length;
         }
     }
-    
+
     out[out_pos] = '\0';
-    
+
     return (char *)out;
 }
 
@@ -374,18 +388,18 @@ irc_color_modifier_cb (void *data, const char *modifier,
                        const char *modifier_data, const char *string)
 {
     int keep_colors;
-    
+
     /* make C compiler happy */
     (void) data;
-    
+
     keep_colors = (modifier_data && (strcmp (modifier_data, "1") == 0)) ? 1 : 0;
-    
+
     if (strcmp (modifier, "irc_color_decode") == 0)
         return irc_color_decode (string, keep_colors);
-    
+
     if (strcmp (modifier, "irc_color_encode") == 0)
         return irc_color_encode (string, keep_colors);
-    
+
     /* unknown modifier */
     return NULL;
 }
