@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2011 Sebastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2012 Sebastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -34,6 +34,7 @@
 #include "irc-channel.h"
 #include "irc-color.h"
 #include "irc-config.h"
+#include "irc-msgbuffer.h"
 #include "irc-nick.h"
 #include "irc-protocol.h"
 #include "irc-server.h"
@@ -59,13 +60,13 @@ const char *
 irc_ctcp_get_default_reply (const char *ctcp)
 {
     int i;
-    
+
     for (i = 0; irc_ctcp_default_reply[i].name; i++)
     {
         if (weechat_strcasecmp (irc_ctcp_default_reply[i].name, ctcp) == 0)
             return irc_ctcp_default_reply[i].reply;
     }
-    
+
     /* unknown CTCP */
     return NULL;
 }
@@ -81,21 +82,21 @@ irc_ctcp_get_reply (struct t_irc_server *server, const char *ctcp)
     char option_name[512];
 
     snprintf (option_name, sizeof (option_name), "%s.%s", server->name, ctcp);
-    
+
     /* search for CTCP in config file, for server */
     ptr_option = weechat_config_search_option (irc_config_file,
                                                irc_config_section_ctcp,
                                                option_name);
     if (ptr_option)
         return weechat_config_string (ptr_option);
-    
+
     /* search for CTCP in config file */
     ptr_option = weechat_config_search_option (irc_config_file,
                                                irc_config_section_ctcp,
                                                ctcp);
     if (ptr_option)
         return weechat_config_string (ptr_option);
-    
+
     /*
      * no CTCP reply found in config, then return default reply, or NULL
      * for unknown CTCP
@@ -119,8 +120,10 @@ irc_ctcp_display_request (struct t_irc_server *server,
     if (reply && !reply[0]
         && !weechat_config_boolean (irc_config_look_display_ctcp_blocked))
         return;
-    
-    weechat_printf_tags ((channel) ? channel->buffer : server->buffer,
+
+    weechat_printf_tags (irc_msgbuffer_get_target_buffer (server, nick,
+                                                          NULL, "ctcp",
+                                                          (channel) ? channel->buffer : NULL),
                          irc_protocol_tags (command, "irc_ctcp", NULL),
                          _("%sCTCP requested by %s%s%s: %s%s%s%s%s%s"),
                          weechat_prefix ("network"),
@@ -147,13 +150,13 @@ irc_ctcp_display_reply_from_nick (struct t_irc_server *server,
     char *pos_end, *pos_space, *pos_args, *pos_usec;
     struct timeval tv;
     long sec1, usec1, sec2, usec2, difftime;
-    
+
     while (arguments && arguments[0])
     {
         pos_end = strchr (arguments + 1, '\01');
         if (pos_end)
             pos_end[0] = '\0';
-        
+
         pos_space = strchr (arguments + 1, ' ');
         if (pos_space)
         {
@@ -169,17 +172,20 @@ irc_ctcp_display_reply_from_nick (struct t_irc_server *server,
                 if (pos_usec)
                 {
                     pos_usec[0] = '\0';
-                    
+
                     gettimeofday (&tv, NULL);
                     sec1 = atol (pos_args);
                     usec1 = atol (pos_usec + 1);
                     sec2 = tv.tv_sec;
                     usec2 = tv.tv_usec;
-                    
+
                     difftime = ((sec2 * 1000000) + usec2) -
                         ((sec1 * 1000000) + usec1);
-                    
-                    weechat_printf_tags (server->buffer,
+                    weechat_printf_tags (irc_msgbuffer_get_target_buffer (server,
+                                                                          nick,
+                                                                          NULL,
+                                                                          "ctcp",
+                                                                          NULL),
                                          irc_protocol_tags (command,
                                                             "irc_ctcp",
                                                             NULL),
@@ -196,13 +202,17 @@ irc_ctcp_display_reply_from_nick (struct t_irc_server *server,
                                          (difftime % 1000000) / 1000,
                                          (NG_("second", "seconds",
                                               (difftime / 1000000))));
-                    
+
                     pos_usec[0] = ' ';
                 }
             }
             else
             {
-                weechat_printf_tags (server->buffer,
+                weechat_printf_tags (irc_msgbuffer_get_target_buffer (server,
+                                                                      nick,
+                                                                      NULL,
+                                                                      "ctcp",
+                                                                      NULL),
                                      irc_protocol_tags (command,
                                                         "irc_ctcp",
                                                         NULL),
@@ -221,7 +231,9 @@ irc_ctcp_display_reply_from_nick (struct t_irc_server *server,
         }
         else
         {
-            weechat_printf_tags (server->buffer,
+            weechat_printf_tags (irc_msgbuffer_get_target_buffer (server, nick,
+                                                                  NULL, "ctcp",
+                                                                  NULL),
                                  irc_protocol_tags (command, NULL, NULL),
                                  _("%sCTCP reply from %s%s%s: %s%s%s%s%s"),
                                  weechat_prefix ("network"),
@@ -234,10 +246,10 @@ irc_ctcp_display_reply_from_nick (struct t_irc_server *server,
                                  "",
                                  "");
         }
-        
+
         if (pos_end)
             pos_end[0] = '\01';
-        
+
         arguments = (pos_end) ? pos_end + 1 : NULL;
     }
 }
@@ -257,7 +269,7 @@ irc_ctcp_reply_to_nick (struct t_irc_server *server,
     int number;
     char hash_key[32];
     const char *str_args;
-    
+
     hashtable = irc_server_sendf (server,
                                   IRC_SERVER_SEND_OUTQ_PRIO_LOW | IRC_SERVER_SEND_RETURN_HASHTABLE,
                                   NULL,
@@ -265,7 +277,7 @@ irc_ctcp_reply_to_nick (struct t_irc_server *server,
                                   nick, ctcp,
                                   (arguments) ? " " : "",
                                   (arguments) ? arguments : "");
-    
+
     if (hashtable)
     {
         if (weechat_config_boolean (irc_config_look_display_ctcp_reply))
@@ -277,7 +289,11 @@ irc_ctcp_reply_to_nick (struct t_irc_server *server,
                 str_args = weechat_hashtable_get (hashtable, hash_key);
                 if (!str_args)
                     break;
-                weechat_printf_tags ((channel) ? channel->buffer : server->buffer,
+                weechat_printf_tags (irc_msgbuffer_get_target_buffer (server,
+                                                                      nick,
+                                                                      NULL,
+                                                                      "ctcp",
+                                                                      (channel) ? channel->buffer : NULL),
                                      irc_protocol_tags (command,
                                                         "irc_ctcp,irc_ctcp_reply,"
                                                         "notify_none,no_highlight",
@@ -311,7 +327,7 @@ irc_ctcp_replace_variables (struct t_irc_server *server, const char *format)
     time_t now;
     char buf[1024];
     struct utsname *buf_uname;
-    
+
     /* clientinfo */
     temp = weechat_string_replace (format, "$clientinfo",
                                    "ACTION DCC CLIENTINFO FINGER PING SOURCE "
@@ -319,7 +335,7 @@ irc_ctcp_replace_variables (struct t_irc_server *server, const char *format)
     if (!temp)
         return NULL;
     res = temp;
-    
+
     /* version */
     info = weechat_info_get ("version", "");
     temp = weechat_string_replace (res, "$version", info);
@@ -327,7 +343,7 @@ irc_ctcp_replace_variables (struct t_irc_server *server, const char *format)
     if (!temp)
         return NULL;
     res = temp;
-    
+
     /* compilation date */
     info = weechat_info_get ("date", "");
     temp = weechat_string_replace (res, "$compilation", info);
@@ -335,7 +351,7 @@ irc_ctcp_replace_variables (struct t_irc_server *server, const char *format)
     if (!temp)
         return NULL;
     res = temp;
-    
+
     /* info about OS */
     buf_uname = (struct utsname *)malloc (sizeof (struct utsname));
     if (buf_uname && (uname (buf_uname) >= 0))
@@ -350,7 +366,7 @@ irc_ctcp_replace_variables (struct t_irc_server *server, const char *format)
             return NULL;
         res = temp;
     }
-    
+
     /* site */
     info = weechat_info_get ("weechat_site", "");
     temp = weechat_string_replace (res, "$site", info);
@@ -358,7 +374,7 @@ irc_ctcp_replace_variables (struct t_irc_server *server, const char *format)
     if (!temp)
         return NULL;
     res = temp;
-    
+
     /* site (download page) */
     info = weechat_info_get ("weechat_site_download", "");
     temp = weechat_string_replace (res, "$download", info);
@@ -366,7 +382,7 @@ irc_ctcp_replace_variables (struct t_irc_server *server, const char *format)
     if (!temp)
         return NULL;
     res = temp;
-    
+
     /* time */
     now = time (NULL);
     snprintf (buf, sizeof (buf), "%s", ctime (&now));
@@ -376,7 +392,7 @@ irc_ctcp_replace_variables (struct t_irc_server *server, const char *format)
     if (!temp)
         return NULL;
     res = temp;
-    
+
     /* username */
     temp = weechat_string_replace (res, "$username",
                                    IRC_SERVER_OPTION_STRING(server, IRC_SERVER_OPTION_USERNAME));
@@ -384,7 +400,7 @@ irc_ctcp_replace_variables (struct t_irc_server *server, const char *format)
     if (!temp)
         return NULL;
     res = temp;
-    
+
     /* realname */
     temp = weechat_string_replace (res, "$realname",
                                    IRC_SERVER_OPTION_STRING(server, IRC_SERVER_OPTION_REALNAME));
@@ -392,7 +408,7 @@ irc_ctcp_replace_variables (struct t_irc_server *server, const char *format)
     if (!temp)
         return NULL;
     res = temp;
-    
+
     /* return result */
     return res;
 }
@@ -429,10 +445,10 @@ irc_ctcp_recv_dcc (struct t_irc_server *server, const char *nick,
     struct t_infolist *infolist;
     struct t_infolist_item *item;
     char charset_modifier[256];
-    
+
     if (!arguments || !arguments[0])
         return;
-    
+
     if (strncmp (arguments, "SEND ", 5) == 0)
     {
         arguments += 5;
@@ -441,7 +457,7 @@ irc_ctcp_recv_dcc (struct t_irc_server *server, const char *nick,
             arguments++;
         }
         dcc_args = strdup (arguments);
-        
+
         if (!dcc_args)
         {
             weechat_printf (server->buffer,
@@ -451,14 +467,14 @@ irc_ctcp_recv_dcc (struct t_irc_server *server, const char *nick,
                             IRC_PLUGIN_NAME, "privmsg");
             return;
         }
-        
+
         /* DCC filename */
         pos_file = dcc_args;
         while (pos_file[0] == ' ')
         {
             pos_file++;
         }
-        
+
         /* look for file size */
         pos_size = strrchr (pos_file, ' ');
         if (!pos_size)
@@ -470,7 +486,7 @@ irc_ctcp_recv_dcc (struct t_irc_server *server, const char *nick,
             free (dcc_args);
             return;
         }
-        
+
         pos = pos_size;
         pos_size++;
         while (pos[0] == ' ')
@@ -478,7 +494,7 @@ irc_ctcp_recv_dcc (struct t_irc_server *server, const char *nick,
             pos--;
         }
         pos[1] = '\0';
-        
+
         /* look for DCC port */
         pos_port = strrchr (pos_file, ' ');
         if (!pos_port)
@@ -490,7 +506,7 @@ irc_ctcp_recv_dcc (struct t_irc_server *server, const char *nick,
             free (dcc_args);
             return;
         }
-        
+
         pos = pos_port;
         pos_port++;
         while (pos[0] == ' ')
@@ -498,7 +514,7 @@ irc_ctcp_recv_dcc (struct t_irc_server *server, const char *nick,
             pos--;
         }
         pos[1] = '\0';
-        
+
         /* look for DCC IP address */
         pos_addr = strrchr (pos_file, ' ');
         if (!pos_addr)
@@ -510,7 +526,7 @@ irc_ctcp_recv_dcc (struct t_irc_server *server, const char *nick,
             free (dcc_args);
             return;
         }
-        
+
         pos = pos_addr;
         pos_addr++;
         while (pos[0] == ' ')
@@ -518,10 +534,10 @@ irc_ctcp_recv_dcc (struct t_irc_server *server, const char *nick,
             pos--;
         }
         pos[1] = '\0';
-        
+
         /* remove double quotes around filename */
         filename = irc_ctcp_dcc_filename_without_quotes (pos_file);
-        
+
         /* add DCC file via xfer plugin */
         infolist = weechat_infolist_new ();
         if (infolist)
@@ -548,14 +564,14 @@ irc_ctcp_recv_dcc (struct t_irc_server *server, const char *nick,
             }
             weechat_infolist_free (infolist);
         }
-        
+
         weechat_hook_signal_send ("irc_dcc",
                                   WEECHAT_HOOK_SIGNAL_STRING,
                                   message);
-        
+
         if (filename)
             free (filename);
-        
+
         free (dcc_args);
     }
     else if (strncmp (arguments, "RESUME ", 7) == 0)
@@ -566,7 +582,7 @@ irc_ctcp_recv_dcc (struct t_irc_server *server, const char *nick,
             arguments++;
         }
         dcc_args = strdup (arguments);
-        
+
         if (!dcc_args)
         {
             weechat_printf (server->buffer,
@@ -576,14 +592,14 @@ irc_ctcp_recv_dcc (struct t_irc_server *server, const char *nick,
                             IRC_PLUGIN_NAME, "privmsg");
             return;
         }
-        
+
         /* DCC filename */
         pos_file = dcc_args;
         while (pos_file[0] == ' ')
         {
             pos_file++;
         }
-        
+
         /* look for resume start position */
         pos_start_resume = strrchr (pos_file, ' ');
         if (!pos_start_resume)
@@ -602,7 +618,7 @@ irc_ctcp_recv_dcc (struct t_irc_server *server, const char *nick,
             pos--;
         }
         pos[1] = '\0';
-        
+
         /* look for DCC port */
         pos_port = strrchr (pos_file, ' ');
         if (!pos_port)
@@ -621,10 +637,10 @@ irc_ctcp_recv_dcc (struct t_irc_server *server, const char *nick,
             pos--;
         }
         pos[1] = '\0';
-        
+
         /* remove double quotes around filename */
         filename = irc_ctcp_dcc_filename_without_quotes (pos_file);
-        
+
         /* accept resume via xfer plugin */
         infolist = weechat_infolist_new ();
         if (infolist)
@@ -645,14 +661,14 @@ irc_ctcp_recv_dcc (struct t_irc_server *server, const char *nick,
             }
             weechat_infolist_free (infolist);
         }
-        
+
         weechat_hook_signal_send ("irc_dcc",
                                   WEECHAT_HOOK_SIGNAL_STRING,
                                   message);
-        
+
         if (filename)
             free (filename);
-        
+
         free (dcc_args);
     }
     else if (strncmp (arguments, "ACCEPT ", 7) == 0)
@@ -663,7 +679,7 @@ irc_ctcp_recv_dcc (struct t_irc_server *server, const char *nick,
             arguments++;
         }
         dcc_args = strdup (arguments);
-        
+
         if (!dcc_args)
         {
             weechat_printf (server->buffer,
@@ -673,14 +689,14 @@ irc_ctcp_recv_dcc (struct t_irc_server *server, const char *nick,
                             "privmsg");
             return;
         }
-        
+
         /* DCC filename */
         pos_file = dcc_args;
         while (pos_file[0] == ' ')
         {
             pos_file++;
         }
-        
+
         /* look for resume start position */
         pos_start_resume = strrchr (pos_file, ' ');
         if (!pos_start_resume)
@@ -699,7 +715,7 @@ irc_ctcp_recv_dcc (struct t_irc_server *server, const char *nick,
             pos--;
         }
         pos[1] = '\0';
-        
+
         /* look for DCC port */
         pos_port = strrchr (pos_file, ' ');
         if (!pos_port)
@@ -718,10 +734,10 @@ irc_ctcp_recv_dcc (struct t_irc_server *server, const char *nick,
             pos--;
         }
         pos[1] = '\0';
-        
+
         /* remove double quotes around filename */
         filename = irc_ctcp_dcc_filename_without_quotes (pos_file);
-        
+
         /* resume file via xfer plugin */
         infolist = weechat_infolist_new ();
         if (infolist)
@@ -742,14 +758,14 @@ irc_ctcp_recv_dcc (struct t_irc_server *server, const char *nick,
             }
             weechat_infolist_free (infolist);
         }
-        
+
         weechat_hook_signal_send ("irc_dcc",
                                   WEECHAT_HOOK_SIGNAL_STRING,
                                   message);
-        
+
         if (filename)
             free (filename);
-        
+
         free (dcc_args);
     }
     else if (strncmp (arguments, "CHAT ", 5) == 0)
@@ -760,7 +776,7 @@ irc_ctcp_recv_dcc (struct t_irc_server *server, const char *nick,
             arguments++;
         }
         dcc_args = strdup (arguments);
-        
+
         if (!dcc_args)
         {
             weechat_printf (server->buffer,
@@ -770,14 +786,14 @@ irc_ctcp_recv_dcc (struct t_irc_server *server, const char *nick,
                             "privmsg");
             return;
         }
-        
+
         /* CHAT type */
         pos_file = dcc_args;
         while (pos_file[0] == ' ')
         {
             pos_file++;
         }
-        
+
         /* DCC IP address */
         pos_addr = strchr (pos_file, ' ');
         if (!pos_addr)
@@ -795,7 +811,7 @@ irc_ctcp_recv_dcc (struct t_irc_server *server, const char *nick,
         {
             pos_addr++;
         }
-        
+
         /* look for DCC port */
         pos_port = strchr (pos_addr, ' ');
         if (!pos_port)
@@ -813,7 +829,7 @@ irc_ctcp_recv_dcc (struct t_irc_server *server, const char *nick,
         {
             pos_port++;
         }
-        
+
         if (weechat_strcasecmp (pos_file, "chat") != 0)
         {
             weechat_printf (server->buffer,
@@ -828,7 +844,7 @@ irc_ctcp_recv_dcc (struct t_irc_server *server, const char *nick,
             free (dcc_args);
             return;
         }
-        
+
         /* add DCC chat via xfer plugin */
         infolist = weechat_infolist_new ();
         if (infolist)
@@ -854,11 +870,11 @@ irc_ctcp_recv_dcc (struct t_irc_server *server, const char *nick,
             }
             weechat_infolist_free (infolist);
         }
-        
+
         weechat_hook_signal_send ("irc_dcc",
                                   WEECHAT_HOOK_SIGNAL_STRING,
                                   message);
-        
+
         free (dcc_args);
     }
 }
@@ -879,13 +895,13 @@ irc_ctcp_recv (struct t_irc_server *server, const char *command,
     struct t_irc_channel *ptr_channel;
     struct t_irc_nick *ptr_nick;
     int nick_is_me;
-    
+
     while (arguments && arguments[0])
     {
         pos_end = strrchr (arguments + 1, '\01');
         if (pos_end)
             pos_end[0] = '\0';
-        
+
         pos_args = NULL;
         pos_space = strchr (arguments + 1, ' ');
         if (pos_space)
@@ -897,25 +913,26 @@ irc_ctcp_recv (struct t_irc_server *server, const char *command,
                 pos_args++;
             }
         }
-        
+
         /* CTCP ACTION */
         if (strcmp (arguments + 1, "ACTION") == 0)
         {
+            nick_is_me = (irc_server_strcasecmp (server, server->nick, nick) == 0);
             if (channel)
             {
-                ptr_nick = irc_nick_search (channel, nick);
-                
+                ptr_nick = irc_nick_search (server, channel, nick);
                 irc_channel_nick_speaking_add (channel,
                                                nick,
                                                (pos_args) ?
                                                weechat_string_has_highlight (pos_args,
                                                                              server->nick) : 0);
                 irc_channel_nick_speaking_time_remove_old (channel);
-                irc_channel_nick_speaking_time_add (channel, nick,
+                irc_channel_nick_speaking_time_add (server, channel, nick,
                                                     time (NULL));
-                
                 weechat_printf_tags (channel->buffer,
                                      irc_protocol_tags (command,
+                                                        (nick_is_me) ?
+                                                        "irc_action,notify_none,no_highlight" :
                                                         "irc_action,notify_message",
                                                         nick),
                                      "%s%s%s%s%s%s",
@@ -928,7 +945,6 @@ irc_ctcp_recv (struct t_irc_server *server, const char *command,
             }
             else
             {
-                nick_is_me = (strcmp (server->nick, nick) == 0);
                 ptr_channel = irc_channel_search (server, remote_nick);
                 if (!ptr_channel)
                 {
@@ -948,7 +964,7 @@ irc_ctcp_recv (struct t_irc_server *server, const char *command,
                 {
                     if (!ptr_channel->topic)
                         irc_channel_set_topic (ptr_channel, address);
-                    
+
                     weechat_printf_tags (ptr_channel->buffer,
                                          irc_protocol_tags (command,
                                                             (nick_is_me) ?
@@ -994,7 +1010,7 @@ irc_ctcp_recv (struct t_irc_server *server, const char *command,
             {
                 irc_ctcp_display_request (server, command, channel, nick,
                                           arguments + 1, pos_args, reply);
-                
+
                 if (reply[0])
                 {
                     decoded_reply = irc_ctcp_replace_variables (server, reply);
@@ -1010,7 +1026,11 @@ irc_ctcp_recv (struct t_irc_server *server, const char *command,
             {
                 if (weechat_config_boolean (irc_config_look_display_ctcp_unknown))
                 {
-                    weechat_printf_tags ((channel) ? channel->buffer : server->buffer,
+                    weechat_printf_tags (irc_msgbuffer_get_target_buffer (server,
+                                                                          nick,
+                                                                          NULL,
+                                                                          "ctcp",
+                                                                          (channel) ? channel->buffer : NULL),
                                          irc_protocol_tags (command,
                                                             "irc_ctcp",
                                                             NULL),
@@ -1028,17 +1048,17 @@ irc_ctcp_recv (struct t_irc_server *server, const char *command,
                 }
             }
         }
-        
+
         weechat_hook_signal_send ("irc_ctcp",
                                   WEECHAT_HOOK_SIGNAL_STRING,
                                   message);
-        
+
         if (pos_space)
             pos_space[0] = ' ';
-        
+
         if (pos_end)
             pos_end[0] = '\01';
-        
+
         arguments = (pos_end) ? pos_end + 1 : NULL;
     }
 }
