@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2011 Sebastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2012 Sebastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -42,6 +42,7 @@ enum t_irc_server_option
     IRC_SERVER_OPTION_SSL_DHKEY_SIZE, /* Diffie Hellman key size             */
     IRC_SERVER_OPTION_SSL_VERIFY,    /* check if the connection is trusted   */
     IRC_SERVER_OPTION_PASSWORD,      /* password for server                  */
+    IRC_SERVER_OPTION_CAPABILITIES,  /* client capabilities to enable        */
     IRC_SERVER_OPTION_SASL_MECHANISM,/* mechanism for SASL authentication    */
     IRC_SERVER_OPTION_SASL_USERNAME, /* username for SASL authentication     */
     IRC_SERVER_OPTION_SASL_PASSWORD, /* password for SASL authentication     */
@@ -102,6 +103,16 @@ enum t_irc_server_option
 #define IRC_SERVER_SEND_OUTQ_PRIO_LOW    2
 #define IRC_SERVER_SEND_RETURN_HASHTABLE 4
 
+/* casemapping (string comparisons for nicks/channels) */
+enum t_irc_server_casemapping
+{
+    IRC_SERVER_CASEMAPPING_RFC1459 = 0,    /* A-Z [ \ ] ^ ==> a-z { | } ~    */
+    IRC_SERVER_CASEMAPPING_STRICT_RFC1459, /* A-Z [ \ ]   ==> a-z { | }      */
+    IRC_SERVER_CASEMAPPING_ASCII,          /* A-Z         ==> a-z            */
+    /* number of casemapping */
+    IRC_SERVER_NUM_CASEMAPPING,
+};
+
 /* output queue of messages to server (for sending slowly to server) */
 
 struct t_irc_outqueue
@@ -121,7 +132,7 @@ struct t_irc_server
     /* user choices */
     char *name;                             /* internal name of server       */
     struct t_config_option *options[IRC_SERVER_NUM_OPTIONS];
-    
+
     /* internal vars */
     int temp_server;                /* temporary server (not saved)          */
     int reloading_from_config;      /* 1 if reloading from config file       */
@@ -153,9 +164,11 @@ struct t_irc_server
     char *nick;                     /* current nickname                      */
     char *nick_modes;               /* nick modes                            */
     char *isupport;                 /* copy of message 005 (ISUPPORT)        */
-    char *prefix_modes;             /* prefix modes from msg 005 (eg "aohv") */
-    char *prefix_chars;             /* prefix chars from msg 005 (eg "&@%+") */
+    char *prefix_modes;             /* prefix modes from msg 005 (eg "ohv")  */
+    char *prefix_chars;             /* prefix chars from msg 005 (eg "@%+")  */
     int nick_max_length;            /* max lenth of nick (from msg 005)      */
+    int casemapping;                /* casemapping from msg 005              */
+    char *chantypes;                /* chantypes from msg 005 (eg "&#")      */
     int reconnect_delay;            /* current reconnect delay (growing)     */
     time_t reconnect_start;         /* this time + delay = reconnect time    */
     time_t command_time;            /* this time + command_delay = time to   */
@@ -208,6 +221,12 @@ extern char *irc_server_option_default[];
 
 extern int irc_server_valid (struct t_irc_server *server);
 extern int irc_server_search_option (const char *option_name);
+extern int irc_server_search_casemapping (const char *casemapping);
+extern int irc_server_strcasecmp (struct t_irc_server *server,
+                                  const char *string1, const char *string2);
+extern int irc_server_strncasecmp (struct t_irc_server *server,
+                                   const char *string1, const char *string2,
+                                   int max);
 extern int irc_server_sasl_enabled (struct t_irc_server *server);
 extern char *irc_server_get_name_without_port (const char *name);
 extern void irc_server_set_addresses (struct t_irc_server *server,
@@ -230,7 +249,7 @@ extern char irc_server_get_prefix_mode_for_char (struct t_irc_server *server,
 extern char irc_server_get_prefix_char_for_mode (struct t_irc_server *server,
                                                  char mode);
 extern struct t_irc_server *irc_server_alloc (const char *name);
-extern int irc_server_alloc_with_url (const char *irc_url);
+extern struct t_irc_server *irc_server_alloc_with_url (const char *irc_url);
 extern void irc_server_apply_command_line_options (struct t_irc_server *server,
                                                    int argc, char **argv);
 extern void irc_server_free_all ();
@@ -250,7 +269,7 @@ extern struct t_irc_server *irc_server_search (const char *server_name);
 extern void irc_server_set_buffer_title (struct t_irc_server *server);
 extern struct t_gui_buffer *irc_server_create_buffer (struct t_irc_server *server);
 extern int irc_server_connect (struct t_irc_server *server);
-extern void irc_server_auto_connect ();
+extern void irc_server_auto_connect (int auto_connect);
 extern void irc_server_autojoin_channels ();
 extern int irc_server_recv_cb (void *data, int fd);
 extern int irc_server_timer_sasl_cb (void *data, int remaining_calls);
