@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2011 Sebastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2012 Sebastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -34,12 +34,13 @@ extern struct t_weechat_plugin *last_weechat_plugin;
 extern int plugin_valid (struct t_weechat_plugin *plugin);
 extern struct t_weechat_plugin *plugin_search (const char *name);
 extern const char *plugin_get_name (struct t_weechat_plugin *plugin);
-extern struct t_weechat_plugin *plugin_load (const char *filename);
-extern void plugin_auto_load ();
+extern struct t_weechat_plugin *plugin_load (const char *filename,
+                                             int argc, char **argv);
+extern void plugin_auto_load (int argc, char **argv);
 extern void plugin_unload (struct t_weechat_plugin *plugin);
 extern void plugin_unload_name (const char *name);
 extern void plugin_unload_all ();
-extern void plugin_reload_name (const char *name);
+extern void plugin_reload_name (const char *name, int argc, char **argv);
 extern void plugin_init (int auto_load, int argc, char *argv[]);
 extern void plugin_end ();
 extern struct t_hdata *plugin_hdata_plugin_cb (void *data,
