@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2011 Sebastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2012 Sebastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -34,8 +34,8 @@
 #include "../weechat-plugin.h"
 #include "relay.h"
 #include "relay-client.h"
-#include "relay-client-irc.h"
-#include "relay-client-weechat.h"
+#include "irc/relay-irc.h"
+#include "weechat/relay-weechat.h"
 #include "relay-config.h"
 #include "relay-buffer.h"
 #include "relay-server.h"
@@ -61,17 +61,17 @@ int
 relay_client_valid (struct t_relay_client *client)
 {
     struct t_relay_client *ptr_client;
-    
+
     if (!client)
         return 0;
-    
+
     for (ptr_client = relay_clients; ptr_client;
          ptr_client = ptr_client->next_client)
     {
         if (ptr_client == client)
             return 1;
     }
-    
+
     /* client not found */
     return 0;
 }
@@ -85,7 +85,7 @@ relay_client_search_by_number (int number)
 {
     struct t_relay_client *ptr_client;
     int i;
-    
+
     i = 0;
     for (ptr_client = relay_clients; ptr_client;
          ptr_client = ptr_client->next_client)
@@ -94,7 +94,7 @@ relay_client_search_by_number (int number)
             return ptr_client;
         i++;
     }
-    
+
     /* client not found */
     return NULL;
 }
@@ -109,12 +109,12 @@ relay_client_recv_cb (void *arg_client, int fd)
     struct t_relay_client *client;
     static char buffer[4096 + 2];
     int num_read;
-    
+
     /* make C compiler happy */
     (void) fd;
-    
+
     client = (struct t_relay_client *)arg_client;
-    
+
     num_read = recv (client->sock, buffer, sizeof (buffer) - 1, 0);
     if (num_read > 0)
     {
@@ -123,10 +123,10 @@ relay_client_recv_cb (void *arg_client, int fd)
         switch (client->protocol)
         {
             case RELAY_PROTOCOL_WEECHAT:
-                relay_client_weechat_recv (client, buffer);
+                relay_weechat_recv (client, buffer);
                 break;
             case RELAY_PROTOCOL_IRC:
-                relay_client_irc_recv (client, buffer);
+                relay_irc_recv (client, buffer);
                 break;
             case RELAY_NUM_PROTOCOLS:
                 break;
@@ -137,7 +137,7 @@ relay_client_recv_cb (void *arg_client, int fd)
     {
         relay_client_set_status (client, RELAY_STATUS_DISCONNECTED);
     }
-    
+
     return WEECHAT_RC_OK;
 }
 
@@ -146,10 +146,10 @@ relay_client_recv_cb (void *arg_client, int fd)
  */
 
 struct t_relay_client *
-relay_client_new (int sock, char *address, struct t_relay_server *server)
+relay_client_new (int sock, const char *address, struct t_relay_server *server)
 {
     struct t_relay_client *new_client;
-    
+
     new_client = malloc (sizeof (*new_client));
     if (new_client)
     {
@@ -158,7 +158,7 @@ relay_client_new (int sock, char *address, struct t_relay_server *server)
         new_client->address = strdup ((address) ? address : "?");
         new_client->status = RELAY_STATUS_CONNECTED;
         new_client->protocol = server->protocol;
-        new_client->protocol_args = strdup (server->protocol_args);
+        new_client->protocol_args = (server->protocol_args) ? strdup (server->protocol_args) : NULL;
         new_client->listen_start_time = server->start_time;
         new_client->start_time = time (NULL);
         new_client->end_time = 0;
@@ -166,20 +166,20 @@ relay_client_new (int sock, char *address, struct t_relay_server *server)
         new_client->last_activity = new_client->start_time;
         new_client->bytes_recv = 0;
         new_client->bytes_sent = 0;
-        
+
         new_client->protocol_data = NULL;
         switch (new_client->protocol)
         {
             case RELAY_PROTOCOL_WEECHAT:
-                relay_client_weechat_alloc (new_client);
+                relay_weechat_alloc (new_client);
                 break;
             case RELAY_PROTOCOL_IRC:
-                relay_client_irc_alloc (new_client);
+                relay_irc_alloc (new_client);
                 break;
             case RELAY_NUM_PROTOCOLS:
                 break;
         }
-        
+
         new_client->prev_client = NULL;
         new_client->next_client = relay_clients;
         if (relay_clients)
@@ -187,9 +187,9 @@ relay_client_new (int sock, char *address, struct t_relay_server *server)
         else
             last_relay_client = new_client;
         relay_clients = new_client;
-        
+
         weechat_printf (NULL,
-                        _("%s: new client from %s%s%s on port %d (id: %d, relaying: %s.%s)"),
+                        _("%s: new client from %s%s%s on port %d (id: %d, relaying: %s%s%s)"),
                         RELAY_PLUGIN_NAME,
                         RELAY_COLOR_CHAT_HOST,
                         new_client->address,
@@ -197,13 +197,14 @@ relay_client_new (int sock, char *address, struct t_relay_server *server)
                         server->port,
                         new_client->id,
                         relay_protocol_string[new_client->protocol],
-                        new_client->protocol_args);
-        
+                        (new_client->protocol_args) ? "." : "",
+                        (new_client->protocol_args) ? new_client->protocol_args : "");
+
         new_client->hook_fd = weechat_hook_fd (new_client->sock,
                                                1, 0, 0,
                                                &relay_client_recv_cb,
                                                new_client);
-        
+
         relay_client_count++;
 
         if (!relay_buffer
@@ -211,7 +212,7 @@ relay_client_new (int sock, char *address, struct t_relay_server *server)
         {
             relay_buffer_open ();
         }
-        
+
         relay_buffer_refresh (WEECHAT_HOTLIST_PRIVATE);
     }
     else
@@ -220,7 +221,7 @@ relay_client_new (int sock, char *address, struct t_relay_server *server)
                         _("%s%s: not enough memory for new client"),
                         weechat_prefix ("error"), RELAY_PLUGIN_NAME);
     }
-    
+
     return new_client;
 }
 
@@ -233,11 +234,11 @@ relay_client_set_status (struct t_relay_client *client,
                          enum t_relay_status status)
 {
     client->status = status;
-    
-    if (RELAY_CLIENT_HAS_ENDED(client->status))
+
+    if (RELAY_CLIENT_HAS_ENDED(client))
     {
         client->end_time = time (NULL);
-        
+
         if (client->hook_fd)
         {
             weechat_unhook (client->hook_fd);
@@ -246,9 +247,10 @@ relay_client_set_status (struct t_relay_client *client,
         switch (client->protocol)
         {
             case RELAY_PROTOCOL_WEECHAT:
+                relay_weechat_close_connection (client);
                 break;
             case RELAY_PROTOCOL_IRC:
-                relay_client_irc_close_connection (client);
+                relay_irc_close_connection (client);
                 break;
             case RELAY_NUM_PROTOCOLS:
                 break;
@@ -257,36 +259,38 @@ relay_client_set_status (struct t_relay_client *client,
         {
             case RELAY_STATUS_AUTH_FAILED:
                 weechat_printf (NULL,
-                                _("%s%s: authentication failed with client %s%s%s (%s.%s)"),
+                                _("%s%s: authentication failed with client %s%s%s (%s%s%s)"),
                                 weechat_prefix ("error"),
                                 RELAY_PLUGIN_NAME,
                                 RELAY_COLOR_CHAT_HOST,
                                 client->address,
                                 RELAY_COLOR_CHAT,
                                 relay_protocol_string[client->protocol],
-                                client->protocol_args);
+                                (client->protocol_args) ? "." : "",
+                                (client->protocol_args) ? client->protocol_args : "");
                 break;
             case RELAY_STATUS_DISCONNECTED:
                 weechat_printf (NULL,
-                                _("%s: disconnected from client %s%s%s (%s.%s)"),
+                                _("%s: disconnected from client %s%s%s (%s%s%s)"),
                                 RELAY_PLUGIN_NAME,
                                 RELAY_COLOR_CHAT_HOST,
                                 client->address,
                                 RELAY_COLOR_CHAT,
                                 relay_protocol_string[client->protocol],
-                                client->protocol_args);
+                                (client->protocol_args) ? "." : "",
+                                (client->protocol_args) ? client->protocol_args : "");
                 break;
             default:
                 break;
         }
-        
+
         if (client->sock >= 0)
         {
             close (client->sock);
             client->sock = -1;
         }
     }
-    
+
     relay_buffer_refresh (WEECHAT_HOTLIST_MESSAGE);
 }
 
@@ -298,10 +302,10 @@ void
 relay_client_free (struct t_relay_client *client)
 {
     struct t_relay_client *new_relay_clients;
-    
+
     if (!client)
         return;
-    
+
     /* remove client from list */
     if (last_relay_client == client)
         last_relay_client = client->prev_client;
@@ -314,7 +318,7 @@ relay_client_free (struct t_relay_client *client)
         new_relay_clients = client->next_client;
     if (client->next_client)
         (client->next_client)->prev_client = client->prev_client;
-    
+
     /* free data */
     if (client->address)
         free (client->address);
@@ -327,20 +331,20 @@ relay_client_free (struct t_relay_client *client)
         switch (client->protocol)
         {
             case RELAY_PROTOCOL_WEECHAT:
-                relay_client_weechat_free (client);
+                relay_weechat_free (client);
                 break;
             case RELAY_PROTOCOL_IRC:
-                relay_client_irc_free (client);
+                relay_irc_free (client);
                 break;
             case RELAY_NUM_PROTOCOLS:
                 break;
         }
     }
-    
+
     free (client);
-    
+
     relay_clients = new_relay_clients;
-    
+
     relay_client_count--;
     if (relay_buffer_selected_line >= relay_client_count)
     {
@@ -383,7 +387,7 @@ void
 relay_client_disconnect_all ()
 {
     struct t_relay_client *ptr_client;
-    
+
     for (ptr_client = relay_clients; ptr_client;
          ptr_client = ptr_client->next_client)
     {
@@ -402,10 +406,10 @@ relay_client_add_to_infolist (struct t_infolist *infolist,
 {
     struct t_infolist_item *ptr_item;
     char value[128];
-    
+
     if (!infolist || !client)
         return 0;
-    
+
     ptr_item = weechat_infolist_new_item (infolist);
     if (!ptr_item)
         return 0;
@@ -442,19 +446,19 @@ relay_client_add_to_infolist (struct t_infolist *infolist,
     snprintf (value, sizeof (value), "%lu", client->bytes_sent);
     if (!weechat_infolist_new_var_string (ptr_item, "bytes_sent", value))
         return 0;
-    
+
     switch (client->protocol)
     {
         case RELAY_PROTOCOL_WEECHAT:
-            relay_client_weechat_add_to_infolist (ptr_item, client);
+            relay_weechat_add_to_infolist (ptr_item, client);
             break;
         case RELAY_PROTOCOL_IRC:
-            relay_client_irc_add_to_infolist (ptr_item, client);
+            relay_irc_add_to_infolist (ptr_item, client);
             break;
         case RELAY_NUM_PROTOCOLS:
             break;
     }
-    
+
     return 1;
 }
 
@@ -466,7 +470,7 @@ void
 relay_client_print_log ()
 {
     struct t_relay_client *ptr_client;
-    
+
     for (ptr_client = relay_clients; ptr_client;
          ptr_client = ptr_client->next_client)
     {
@@ -493,10 +497,10 @@ relay_client_print_log ()
         switch (ptr_client->protocol)
         {
             case RELAY_PROTOCOL_WEECHAT:
-                relay_client_weechat_print_log (ptr_client);
+                relay_weechat_print_log (ptr_client);
                 break;
             case RELAY_PROTOCOL_IRC:
-                relay_client_irc_print_log (ptr_client);
+                relay_irc_print_log (ptr_client);
                 break;
             case RELAY_NUM_PROTOCOLS:
                 break;
