@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2011 Sebastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2012 Sebastien Helleu <flashcode@flashtux.org>
  * Copyright (C) 2005-2007 Emmanuel Bouthenot <kolter@openics.org>
  *
  * This file is part of WeeChat, the extensible chat client.
@@ -52,6 +52,39 @@ const char *python_current_script_filename = NULL;
 PyThreadState *python_mainThreadState = NULL;
 char *python2_bin = NULL;
 
+/* outputs subroutines */
+static PyObject *weechat_python_output (PyObject *self, PyObject *args);
+static PyMethodDef weechat_python_output_funcs[] = {
+    { "write", weechat_python_output, METH_VARARGS, "" },
+    { NULL, NULL, 0, NULL }
+};
+
+#if PY_MAJOR_VERSION >= 3
+/* module definition for python >= 3.x */
+static struct PyModuleDef moduleDef = {
+    PyModuleDef_HEAD_INIT,
+    "weechat",
+    NULL,
+    -1,
+    weechat_python_funcs,
+    NULL,
+    NULL,
+    NULL,
+    NULL
+};
+static struct PyModuleDef moduleDefOutputs = {
+    PyModuleDef_HEAD_INIT,
+    "weechatOutputs",
+    NULL,
+    -1,
+    weechat_python_output_funcs,
+    NULL,
+    NULL,
+    NULL,
+    NULL
+};
+#endif
+
 /*
  * string used to execute action "install":
  * when signal "python_install_script" is received, name of string
@@ -83,12 +116,12 @@ weechat_python_set_python2_bin ()
     char *versions[] = { "2", "2.7", "2.6", "2.5", "2.4", "2.3", "2.2", NULL };
     int num_paths, i, j, rc;
     struct stat stat_buf;
-    
+
     python2_bin = NULL;
-    
+
     dir_separator = weechat_info_get ("dir_separator", "");
     path = getenv ("PATH");
-    
+
     if (dir_separator && path)
     {
         paths = weechat_string_split (path, ":", 0, 0, &num_paths);
@@ -114,11 +147,36 @@ weechat_python_set_python2_bin ()
             weechat_string_free_split (paths);
         }
     }
-    
+
     if (!python2_bin)
         python2_bin = strdup ("python");
 }
 
+/*
+ * weechat_python_unicode_to_string: convert a python unicode to a C UTF-8
+ *                                   string
+ *                                   Note: result has to be freed after use
+ */
+
+char *
+weechat_python_unicode_to_string (PyObject *obj)
+{
+    PyObject *utf8string;
+    char *str;
+
+    str = NULL;
+
+    utf8string = PyUnicode_AsUTF8String (obj);
+    if (utf8string)
+    {
+        if (PyBytes_AsString (utf8string))
+            str = strdup (PyBytes_AsString (utf8string));
+        Py_XDECREF(utf8string);
+    }
+
+    return str;
+}
+
 /*
  * weechat_python_hashtable_map_cb: callback called for each key/value in a
  *                                  hashtable
@@ -127,21 +185,21 @@ weechat_python_set_python2_bin ()
 void
 weechat_python_hashtable_map_cb (void *data,
                                  struct t_hashtable *hashtable,
-                                 const void *key,
-                                 const void *value)
+                                 const char *key,
+                                 const char *value)
 {
     PyObject *dict, *dict_key, *dict_value;
 
     /* make C compiler happy */
     (void) hashtable;
-    
+
     dict = (PyObject *)data;
-    
-    dict_key = Py_BuildValue ("s", (const char *)key);
-    dict_value = Py_BuildValue ("s", (const char *)value);
-    
+
+    dict_key = Py_BuildValue ("s", key);
+    dict_value = Py_BuildValue ("s", value);
+
     PyDict_SetItem (dict, dict_key, dict_value);
-    
+
     Py_DECREF (dict_key);
     Py_DECREF (dict_value);
 }
@@ -155,18 +213,18 @@ PyObject *
 weechat_python_hashtable_to_dict (struct t_hashtable *hashtable)
 {
     PyObject *dict;
-    
+
     dict = PyDict_New ();
     if (!dict)
     {
         Py_INCREF(Py_None);
         return Py_None;
     }
-    
-    weechat_hashtable_map (hashtable,
-                           &weechat_python_hashtable_map_cb,
-                           dict);
-    
+
+    weechat_hashtable_map_string (hashtable,
+                                  &weechat_python_hashtable_map_cb,
+                                  dict);
+
     return dict;
 }
 
@@ -186,7 +244,7 @@ weechat_python_dict_to_hashtable (PyObject *dict, int hashtable_size)
     PyObject *key, *value;
     Py_ssize_t pos;
     char *str_key, *str_value;
-    
+
     hashtable = weechat_hashtable_new (hashtable_size,
                                        WEECHAT_HASHTABLE_STRING,
                                        WEECHAT_HASHTABLE_STRING,
@@ -194,15 +252,36 @@ weechat_python_dict_to_hashtable (PyObject *dict, int hashtable_size)
                                        NULL);
     if (!hashtable)
         return NULL;
-    
+
     pos = 0;
     while (PyDict_Next (dict, &pos, &key, &value))
     {
-        str_key = PyString_AsString (key);
-        str_value = PyString_AsString (value);
-        weechat_hashtable_set (hashtable, str_key, str_value);
+        str_key = NULL;
+        str_value = NULL;
+        if (PyBytes_Check (key))
+        {
+            if (PyBytes_AsString (key))
+                str_key = strdup (PyBytes_AsString (key));
+        }
+        else
+            str_key = weechat_python_unicode_to_string (key);
+        if (PyBytes_Check (value))
+        {
+            if (PyBytes_AsString (value))
+                str_value = strdup (PyBytes_AsString (value));
+        }
+        else
+            str_value = weechat_python_unicode_to_string (value);
+
+        if (str_key)
+            weechat_hashtable_set (hashtable, str_key, str_value);
+
+        if (str_key)
+            free (str_key);
+        if (str_value)
+            free (str_value);
     }
-    
+
     return hashtable;
 }
 
@@ -217,15 +296,12 @@ weechat_python_exec (struct t_plugin_script *script,
 {
     struct t_plugin_script *old_python_current_script;
     PyThreadState *old_interpreter;
-    PyObject *evMain;
-    PyObject *evDict;
-    PyObject *evFunc;
-    PyObject *rc;
+    PyObject *evMain, *evDict, *evFunc, *rc;
     void *argv2[16], *ret_value;
     int i, argc, *ret_int;
-    
+
     /* PyEval_AcquireLock (); */
-    
+
     old_python_current_script = python_current_script;
     old_interpreter = NULL;
     if (script->interpreter)
@@ -233,11 +309,11 @@ weechat_python_exec (struct t_plugin_script *script,
         old_interpreter = PyThreadState_Swap (NULL);
         PyThreadState_Swap (script->interpreter);
     }
-    
+
     evMain = PyImport_AddModule ((char *) "__main__");
     evDict = PyModule_GetDict (evMain);
     evFunc = PyDict_GetItemString (evDict, function);
-    
+
     if ( !(evFunc && PyCallable_Check (evFunc)) )
     {
         weechat_printf (NULL,
@@ -248,7 +324,7 @@ weechat_python_exec (struct t_plugin_script *script,
             PyThreadState_Swap (old_interpreter);
         return NULL;
     }
-    
+
     python_current_script = script;
 
     if (argv && argv[0])
@@ -272,34 +348,39 @@ weechat_python_exec (struct t_plugin_script *script,
     {
         rc = PyObject_CallFunction (evFunc, NULL);
     }
-    
+
     ret_value = NULL;
-    
+
     /*
      * ugly hack : rc = NULL while 'return weechat.WEECHAT_RC_OK ....
      * because of '#define WEECHAT_RC_OK 0'
      */
     if (rc ==  NULL)
-        rc = PyInt_FromLong (0);
-    
+        rc = PyLong_FromLong ((long)0);
+
     if (PyErr_Occurred())
     {
         PyErr_Print ();
         Py_XDECREF(rc);
     }
-    else if ((ret_type == WEECHAT_SCRIPT_EXEC_STRING) && (PyString_Check (rc)))
+    else if ((ret_type == WEECHAT_SCRIPT_EXEC_STRING) && (PyUnicode_Check (rc)))
     {
-        if (PyString_AsString (rc))
-            ret_value = strdup (PyString_AsString(rc));
+        ret_value = weechat_python_unicode_to_string (rc);
+        Py_XDECREF(rc);
+    }
+    else if ((ret_type == WEECHAT_SCRIPT_EXEC_STRING) && (PyBytes_Check (rc)))
+    {
+        if (PyBytes_AsString (rc))
+            ret_value = strdup (PyBytes_AsString (rc));
         else
             ret_value = NULL;
         Py_XDECREF(rc);
     }
-    else if ((ret_type == WEECHAT_SCRIPT_EXEC_INT) && (PyInt_Check (rc)))
+    else if ((ret_type == WEECHAT_SCRIPT_EXEC_INT) && (PyLong_Check (rc)))
     {
         ret_int = malloc (sizeof (*ret_int));
         if (ret_int)
-            *ret_int = (int) PyInt_AsLong(rc);
+            *ret_int = (int) PyLong_AsLong(rc);
         ret_value = ret_int;
         Py_XDECREF(rc);
     }
@@ -316,21 +397,21 @@ weechat_python_exec (struct t_plugin_script *script,
                                          "a valid value"),
                         weechat_prefix ("error"), PYTHON_PLUGIN_NAME, function);
     }
-    
-    if (ret_value == NULL)
+
+    if (!ret_value)
     {
         weechat_printf (NULL,
                         weechat_gettext ("%s%s: error in function \"%s\""),
                         weechat_prefix ("error"), PYTHON_PLUGIN_NAME, function);
     }
-    
+
     /* PyEval_ReleaseThread (python_current_script->interpreter); */
-    
+
     python_current_script = old_python_current_script;
-    
+
     if (old_interpreter)
         PyThreadState_Swap (old_interpreter);
-    
+
     return ret_value;
 }
 
@@ -342,10 +423,10 @@ static PyObject *
 weechat_python_output (PyObject *self, PyObject *args)
 {
     char *msg, *m, *p;
-    
+
     /* make C compiler happy */
     (void) self;
-    
+
     msg = NULL;
 
     if (!PyArg_ParseTuple (args, "s", &msg))
@@ -385,20 +466,95 @@ weechat_python_output (PyObject *self, PyObject *args)
         else
             strcat (python_buffer_output, m);
     }
-    
+
     Py_INCREF(Py_None);
     return Py_None;
 }
 
 /*
- * Outputs subroutines
+ * weechat_python_init_module_weechat: initialize the "weechat" module
  */
 
-static
-PyMethodDef weechat_python_output_funcs[] = {
-    { "write", weechat_python_output, METH_VARARGS, "" },
-    { NULL, NULL, 0, NULL }
-};
+#if PY_MAJOR_VERSION >= 3
+static PyObject *weechat_python_init_module_weechat ()
+#else
+void weechat_python_init_module_weechat ()
+#endif
+{
+    PyObject *weechat_module, *weechat_dict;
+
+#if PY_MAJOR_VERSION >= 3
+    /* python >= 3.x */
+    weechat_module = PyModule_Create (&moduleDef);
+#else
+    /* python <= 2.x */
+    weechat_module = Py_InitModule ("weechat", weechat_python_funcs);
+#endif
+
+    if (!weechat_module)
+    {
+        weechat_printf (NULL,
+                        weechat_gettext ("%s%s: unable to initialize WeeChat "
+                                         "module"),
+                        weechat_prefix ("error"), PYTHON_PLUGIN_NAME);
+#if PY_MAJOR_VERSION >= 3
+        return NULL;
+#else
+        return;
+#endif
+    }
+
+    /* define some constants */
+    weechat_dict = PyModule_GetDict(weechat_module);
+    PyDict_SetItemString(weechat_dict, "WEECHAT_RC_OK", PyLong_FromLong((long) WEECHAT_RC_OK));
+    PyDict_SetItemString(weechat_dict, "WEECHAT_RC_OK_EAT", PyLong_FromLong((long) WEECHAT_RC_OK_EAT));
+    PyDict_SetItemString(weechat_dict, "WEECHAT_RC_ERROR", PyLong_FromLong((long) WEECHAT_RC_ERROR));
+
+    PyDict_SetItemString(weechat_dict, "WEECHAT_CONFIG_READ_OK", PyLong_FromLong((long) WEECHAT_CONFIG_READ_OK));
+    PyDict_SetItemString(weechat_dict, "WEECHAT_CONFIG_READ_MEMORY_ERROR", PyLong_FromLong((long) WEECHAT_CONFIG_READ_MEMORY_ERROR));
+    PyDict_SetItemString(weechat_dict, "WEECHAT_CONFIG_READ_FILE_NOT_FOUND", PyLong_FromLong((long) WEECHAT_CONFIG_READ_FILE_NOT_FOUND));
+    PyDict_SetItemString(weechat_dict, "WEECHAT_CONFIG_WRITE_OK", PyLong_FromLong((long) WEECHAT_CONFIG_WRITE_OK));
+    PyDict_SetItemString(weechat_dict, "WEECHAT_CONFIG_WRITE_ERROR", PyLong_FromLong((long) WEECHAT_CONFIG_WRITE_ERROR));
+    PyDict_SetItemString(weechat_dict, "WEECHAT_CONFIG_WRITE_MEMORY_ERROR", PyLong_FromLong((long) WEECHAT_CONFIG_WRITE_MEMORY_ERROR));
+    PyDict_SetItemString(weechat_dict, "WEECHAT_CONFIG_OPTION_SET_OK_CHANGED", PyLong_FromLong((long) WEECHAT_CONFIG_OPTION_SET_OK_CHANGED));
+    PyDict_SetItemString(weechat_dict, "WEECHAT_CONFIG_OPTION_SET_OK_SAME_VALUE", PyLong_FromLong((long) WEECHAT_CONFIG_OPTION_SET_OK_SAME_VALUE));
+    PyDict_SetItemString(weechat_dict, "WEECHAT_CONFIG_OPTION_SET_ERROR", PyLong_FromLong((long) WEECHAT_CONFIG_OPTION_SET_ERROR));
+    PyDict_SetItemString(weechat_dict, "WEECHAT_CONFIG_OPTION_SET_OPTION_NOT_FOUND", PyLong_FromLong((long) WEECHAT_CONFIG_OPTION_SET_OPTION_NOT_FOUND));
+    PyDict_SetItemString(weechat_dict, "WEECHAT_CONFIG_OPTION_UNSET_OK_NO_RESET", PyLong_FromLong((long) WEECHAT_CONFIG_OPTION_UNSET_OK_NO_RESET));
+    PyDict_SetItemString(weechat_dict, "WEECHAT_CONFIG_OPTION_UNSET_OK_RESET", PyLong_FromLong((long) WEECHAT_CONFIG_OPTION_UNSET_OK_RESET));
+    PyDict_SetItemString(weechat_dict, "WEECHAT_CONFIG_OPTION_UNSET_OK_REMOVED", PyLong_FromLong((long) WEECHAT_CONFIG_OPTION_UNSET_OK_REMOVED));
+    PyDict_SetItemString(weechat_dict, "WEECHAT_CONFIG_OPTION_UNSET_ERROR", PyLong_FromLong((long) WEECHAT_CONFIG_OPTION_UNSET_ERROR));
+
+    PyDict_SetItemString(weechat_dict, "WEECHAT_LIST_POS_SORT", PyUnicode_FromString(WEECHAT_LIST_POS_SORT));
+    PyDict_SetItemString(weechat_dict, "WEECHAT_LIST_POS_BEGINNING", PyUnicode_FromString(WEECHAT_LIST_POS_BEGINNING));
+    PyDict_SetItemString(weechat_dict, "WEECHAT_LIST_POS_END", PyUnicode_FromString(WEECHAT_LIST_POS_END));
+
+    PyDict_SetItemString(weechat_dict, "WEECHAT_HOTLIST_LOW", PyUnicode_FromString(WEECHAT_HOTLIST_LOW));
+    PyDict_SetItemString(weechat_dict, "WEECHAT_HOTLIST_MESSAGE", PyUnicode_FromString(WEECHAT_HOTLIST_MESSAGE));
+    PyDict_SetItemString(weechat_dict, "WEECHAT_HOTLIST_PRIVATE", PyUnicode_FromString(WEECHAT_HOTLIST_PRIVATE));
+    PyDict_SetItemString(weechat_dict, "WEECHAT_HOTLIST_HIGHLIGHT", PyUnicode_FromString(WEECHAT_HOTLIST_HIGHLIGHT));
+
+    PyDict_SetItemString(weechat_dict, "WEECHAT_HOOK_PROCESS_RUNNING", PyLong_FromLong((long) WEECHAT_HOOK_PROCESS_RUNNING));
+    PyDict_SetItemString(weechat_dict, "WEECHAT_HOOK_PROCESS_ERROR", PyLong_FromLong((long) WEECHAT_HOOK_PROCESS_ERROR));
+
+    PyDict_SetItemString(weechat_dict, "WEECHAT_HOOK_CONNECT_OK", PyLong_FromLong((long) WEECHAT_HOOK_CONNECT_OK));
+    PyDict_SetItemString(weechat_dict, "WEECHAT_HOOK_CONNECT_ADDRESS_NOT_FOUND", PyLong_FromLong((long) WEECHAT_HOOK_CONNECT_ADDRESS_NOT_FOUND));
+    PyDict_SetItemString(weechat_dict, "WEECHAT_HOOK_CONNECT_IP_ADDRESS_NOT_FOUND", PyLong_FromLong((long) WEECHAT_HOOK_CONNECT_IP_ADDRESS_NOT_FOUND));
+    PyDict_SetItemString(weechat_dict, "WEECHAT_HOOK_CONNECT_CONNECTION_REFUSED", PyLong_FromLong((long) WEECHAT_HOOK_CONNECT_CONNECTION_REFUSED));
+    PyDict_SetItemString(weechat_dict, "WEECHAT_HOOK_CONNECT_PROXY_ERROR", PyLong_FromLong((long) WEECHAT_HOOK_CONNECT_PROXY_ERROR));
+    PyDict_SetItemString(weechat_dict, "WEECHAT_HOOK_CONNECT_LOCAL_HOSTNAME_ERROR", PyLong_FromLong((long) WEECHAT_HOOK_CONNECT_LOCAL_HOSTNAME_ERROR));
+    PyDict_SetItemString(weechat_dict, "WEECHAT_HOOK_CONNECT_GNUTLS_INIT_ERROR", PyLong_FromLong((long) WEECHAT_HOOK_CONNECT_GNUTLS_INIT_ERROR));
+    PyDict_SetItemString(weechat_dict, "WEECHAT_HOOK_CONNECT_GNUTLS_HANDSHAKE_ERROR", PyLong_FromLong((long) WEECHAT_HOOK_CONNECT_GNUTLS_HANDSHAKE_ERROR));
+    PyDict_SetItemString(weechat_dict, "WEECHAT_HOOK_CONNECT_MEMORY_ERROR", PyLong_FromLong((long) WEECHAT_HOOK_CONNECT_MEMORY_ERROR));
+
+    PyDict_SetItemString(weechat_dict, "WEECHAT_HOOK_SIGNAL_STRING", PyUnicode_FromString(WEECHAT_HOOK_SIGNAL_STRING));
+    PyDict_SetItemString(weechat_dict, "WEECHAT_HOOK_SIGNAL_INT", PyUnicode_FromString(WEECHAT_HOOK_SIGNAL_INT));
+    PyDict_SetItemString(weechat_dict, "WEECHAT_HOOK_SIGNAL_POINTER", PyUnicode_FromString(WEECHAT_HOOK_SIGNAL_POINTER));
+
+#if PY_MAJOR_VERSION >= 3
+    return weechat_module;
+#endif
+}
 
 /*
  * weechat_python_load: load a Python script
@@ -408,14 +564,16 @@ int
 weechat_python_load (const char *filename)
 {
     char *argv[] = { "__weechat_plugin__" , NULL };
+#if PY_MAJOR_VERSION >= 3
+    wchar_t *wargv[] = { NULL, NULL };
+#endif
     FILE *fp;
     PyThreadState *python_current_interpreter;
-    PyObject *weechat_module, *weechat_outputs, *weechat_dict;
-    PyObject *python_path, *path;
+    PyObject *weechat_outputs, *python_path, *path;
     const char *weechat_home;
-    char *p_home;
+    char *str_home;
     int len;
-    
+
     if ((fp = fopen (filename, "r")) == NULL)
     {
         weechat_printf (NULL,
@@ -423,22 +581,40 @@ weechat_python_load (const char *filename)
                         weechat_prefix ("error"), PYTHON_PLUGIN_NAME, filename);
         return 0;
     }
-    
+
     if ((weechat_python_plugin->debug >= 1) || !python_quiet)
     {
         weechat_printf (NULL,
                         weechat_gettext ("%s: loading script \"%s\""),
                         PYTHON_PLUGIN_NAME, filename);
     }
-    
+
     python_current_script = NULL;
     python_registered_script = NULL;
-    
+
     /* PyEval_AcquireLock (); */
     python_current_interpreter = Py_NewInterpreter ();
+#if PY_MAJOR_VERSION >= 3
+    /* python >= 3.x */
+    len = strlen (argv[0]);
+    wargv[0] = malloc ((len + 1) * sizeof (wargv[0][0]));
+    if (wargv[0])
+    {
+        if (mbstowcs (wargv[0], argv[0], len) == (size_t)(-1))
+        {
+            free (wargv[0]);
+            wargv[0] = NULL;
+        }
+        PySys_SetArgv(1, wargv);
+        if (wargv[0])
+            free (wargv[0]);
+    }
+#else
+    /* python <= 2.x */
     PySys_SetArgv(1, argv);
-    
-    if (python_current_interpreter == NULL)
+#endif
+
+    if (!python_current_interpreter)
     {
         weechat_printf (NULL,
                         weechat_gettext ("%s%s: unable to create new "
@@ -448,95 +624,39 @@ weechat_python_load (const char *filename)
         /* PyEval_ReleaseLock (); */
         return 0;
     }
-    
+
     PyThreadState_Swap (python_current_interpreter);
-    
-    weechat_module = Py_InitModule ("weechat", weechat_python_funcs);
-
-    if ( weechat_module == NULL)
-    {
-        weechat_printf (NULL,
-                        weechat_gettext ("%s%s: unable to initialize WeeChat "
-                                         "module"),
-                        weechat_prefix ("error"), PYTHON_PLUGIN_NAME);
-        fclose (fp);
-        
-        Py_EndInterpreter (python_current_interpreter);
-        /* PyEval_ReleaseLock (); */
-
-        return 0;
-    }
 
     /* adding $weechat_dir/python in $PYTHONPATH */
     python_path = PySys_GetObject ("path");
     weechat_home = weechat_info_get ("weechat_dir", "");
     if (weechat_home)
     {
-        len = strlen (weechat_home) + 1 + strlen(PYTHON_PLUGIN_NAME) + 1;
-        p_home = malloc (len);
-        if (p_home)
+        len = strlen (weechat_home) + 1 + strlen (PYTHON_PLUGIN_NAME) + 1;
+        str_home = malloc (len);
+        if (str_home)
         {
-            snprintf (p_home, len, "%s/python", weechat_home);
-            path = PyString_FromString (p_home);
+            snprintf (str_home, len, "%s/python", weechat_home);
+            path = PyBytes_FromString (str_home);
             if (path != NULL)
             {
                 PyList_Insert (python_path, 0, path);
                 Py_DECREF (path);
             }
-            free (p_home);
+            free (str_home);
         }
     }
-    
-    /* define some constants */
-    weechat_dict = PyModule_GetDict(weechat_module);
-    PyDict_SetItemString(weechat_dict, "WEECHAT_RC_OK", PyInt_FromLong((long) WEECHAT_RC_OK));
-    PyDict_SetItemString(weechat_dict, "WEECHAT_RC_OK_EAT", PyInt_FromLong((long) WEECHAT_RC_OK_EAT));
-    PyDict_SetItemString(weechat_dict, "WEECHAT_RC_ERROR", PyInt_FromLong((long) WEECHAT_RC_ERROR));
-    
-    PyDict_SetItemString(weechat_dict, "WEECHAT_CONFIG_READ_OK", PyInt_FromLong((long) WEECHAT_CONFIG_READ_OK));
-    PyDict_SetItemString(weechat_dict, "WEECHAT_CONFIG_READ_MEMORY_ERROR", PyInt_FromLong((long) WEECHAT_CONFIG_READ_MEMORY_ERROR));
-    PyDict_SetItemString(weechat_dict, "WEECHAT_CONFIG_READ_FILE_NOT_FOUND", PyInt_FromLong((long) WEECHAT_CONFIG_READ_FILE_NOT_FOUND));
-    PyDict_SetItemString(weechat_dict, "WEECHAT_CONFIG_WRITE_OK", PyInt_FromLong((long) WEECHAT_CONFIG_WRITE_OK));
-    PyDict_SetItemString(weechat_dict, "WEECHAT_CONFIG_WRITE_ERROR", PyInt_FromLong((long) WEECHAT_CONFIG_WRITE_ERROR));
-    PyDict_SetItemString(weechat_dict, "WEECHAT_CONFIG_WRITE_MEMORY_ERROR", PyInt_FromLong((long) WEECHAT_CONFIG_WRITE_MEMORY_ERROR));
-    PyDict_SetItemString(weechat_dict, "WEECHAT_CONFIG_OPTION_SET_OK_CHANGED", PyInt_FromLong((long) WEECHAT_CONFIG_OPTION_SET_OK_CHANGED));
-    PyDict_SetItemString(weechat_dict, "WEECHAT_CONFIG_OPTION_SET_OK_SAME_VALUE", PyInt_FromLong((long) WEECHAT_CONFIG_OPTION_SET_OK_SAME_VALUE));
-    PyDict_SetItemString(weechat_dict, "WEECHAT_CONFIG_OPTION_SET_ERROR", PyInt_FromLong((long) WEECHAT_CONFIG_OPTION_SET_ERROR));
-    PyDict_SetItemString(weechat_dict, "WEECHAT_CONFIG_OPTION_SET_OPTION_NOT_FOUND", PyInt_FromLong((long) WEECHAT_CONFIG_OPTION_SET_OPTION_NOT_FOUND));
-    PyDict_SetItemString(weechat_dict, "WEECHAT_CONFIG_OPTION_UNSET_OK_NO_RESET", PyInt_FromLong((long) WEECHAT_CONFIG_OPTION_UNSET_OK_NO_RESET));
-    PyDict_SetItemString(weechat_dict, "WEECHAT_CONFIG_OPTION_UNSET_OK_RESET", PyInt_FromLong((long) WEECHAT_CONFIG_OPTION_UNSET_OK_RESET));
-    PyDict_SetItemString(weechat_dict, "WEECHAT_CONFIG_OPTION_UNSET_OK_REMOVED", PyInt_FromLong((long) WEECHAT_CONFIG_OPTION_UNSET_OK_REMOVED));
-    PyDict_SetItemString(weechat_dict, "WEECHAT_CONFIG_OPTION_UNSET_ERROR", PyInt_FromLong((long) WEECHAT_CONFIG_OPTION_UNSET_ERROR));
-    
-    PyDict_SetItemString(weechat_dict, "WEECHAT_LIST_POS_SORT", PyString_FromString(WEECHAT_LIST_POS_SORT));
-    PyDict_SetItemString(weechat_dict, "WEECHAT_LIST_POS_BEGINNING", PyString_FromString(WEECHAT_LIST_POS_BEGINNING));
-    PyDict_SetItemString(weechat_dict, "WEECHAT_LIST_POS_END", PyString_FromString(WEECHAT_LIST_POS_END));
-    
-    PyDict_SetItemString(weechat_dict, "WEECHAT_HOTLIST_LOW", PyString_FromString(WEECHAT_HOTLIST_LOW));
-    PyDict_SetItemString(weechat_dict, "WEECHAT_HOTLIST_MESSAGE", PyString_FromString(WEECHAT_HOTLIST_MESSAGE));
-    PyDict_SetItemString(weechat_dict, "WEECHAT_HOTLIST_PRIVATE", PyString_FromString(WEECHAT_HOTLIST_PRIVATE));
-    PyDict_SetItemString(weechat_dict, "WEECHAT_HOTLIST_HIGHLIGHT", PyString_FromString(WEECHAT_HOTLIST_HIGHLIGHT));
-    
-    PyDict_SetItemString(weechat_dict, "WEECHAT_HOOK_PROCESS_RUNNING", PyInt_FromLong((long) WEECHAT_HOOK_PROCESS_RUNNING));
-    PyDict_SetItemString(weechat_dict, "WEECHAT_HOOK_PROCESS_ERROR", PyInt_FromLong((long) WEECHAT_HOOK_PROCESS_ERROR));
-    
-    PyDict_SetItemString(weechat_dict, "WEECHAT_HOOK_CONNECT_OK", PyInt_FromLong((long) WEECHAT_HOOK_CONNECT_OK));
-    PyDict_SetItemString(weechat_dict, "WEECHAT_HOOK_CONNECT_ADDRESS_NOT_FOUND", PyInt_FromLong((long) WEECHAT_HOOK_CONNECT_ADDRESS_NOT_FOUND));
-    PyDict_SetItemString(weechat_dict, "WEECHAT_HOOK_CONNECT_IP_ADDRESS_NOT_FOUND", PyInt_FromLong((long) WEECHAT_HOOK_CONNECT_IP_ADDRESS_NOT_FOUND));
-    PyDict_SetItemString(weechat_dict, "WEECHAT_HOOK_CONNECT_CONNECTION_REFUSED", PyInt_FromLong((long) WEECHAT_HOOK_CONNECT_CONNECTION_REFUSED));
-    PyDict_SetItemString(weechat_dict, "WEECHAT_HOOK_CONNECT_PROXY_ERROR", PyInt_FromLong((long) WEECHAT_HOOK_CONNECT_PROXY_ERROR));
-    PyDict_SetItemString(weechat_dict, "WEECHAT_HOOK_CONNECT_LOCAL_HOSTNAME_ERROR", PyInt_FromLong((long) WEECHAT_HOOK_CONNECT_LOCAL_HOSTNAME_ERROR));
-    PyDict_SetItemString(weechat_dict, "WEECHAT_HOOK_CONNECT_GNUTLS_INIT_ERROR", PyInt_FromLong((long) WEECHAT_HOOK_CONNECT_GNUTLS_INIT_ERROR));
-    PyDict_SetItemString(weechat_dict, "WEECHAT_HOOK_CONNECT_GNUTLS_HANDSHAKE_ERROR", PyInt_FromLong((long) WEECHAT_HOOK_CONNECT_GNUTLS_HANDSHAKE_ERROR));
-    PyDict_SetItemString(weechat_dict, "WEECHAT_HOOK_CONNECT_MEMORY_ERROR", PyInt_FromLong((long) WEECHAT_HOOK_CONNECT_MEMORY_ERROR));
-    
-    PyDict_SetItemString(weechat_dict, "WEECHAT_HOOK_SIGNAL_STRING", PyString_FromString(WEECHAT_HOOK_SIGNAL_STRING));
-    PyDict_SetItemString(weechat_dict, "WEECHAT_HOOK_SIGNAL_INT", PyString_FromString(WEECHAT_HOOK_SIGNAL_INT));
-    PyDict_SetItemString(weechat_dict, "WEECHAT_HOOK_SIGNAL_POINTER", PyString_FromString(WEECHAT_HOOK_SIGNAL_POINTER));
-    
+
+#if PY_MAJOR_VERSION >= 3
+    /* python 3.x (or newer) */
+    weechat_outputs = PyModule_Create (&moduleDefOutputs);
+#else
+    /* python 2.x */
     weechat_outputs = Py_InitModule("weechatOutputs",
                                     weechat_python_output_funcs);
-    if (weechat_outputs == NULL)
+#endif
+
+    if (!weechat_outputs)
     {
         weechat_printf (NULL,
                         weechat_gettext ("%s%s: unable to redirect stdout and "
@@ -558,9 +678,9 @@ weechat_python_load (const char *filename)
                             weechat_prefix ("error"), PYTHON_PLUGIN_NAME);
         }
     }
-    
+
     python_current_script_filename = filename;
-    
+
     if (PyRun_SimpleFile (fp, filename) != 0)
     {
         weechat_printf (NULL,
@@ -570,18 +690,18 @@ weechat_python_load (const char *filename)
 
         if (PyErr_Occurred ())
             PyErr_Print ();
-        
-        /* if script was registered, removing from list */
+
+        /* if script was registered, remove it from list */
         if (python_current_script != NULL)
         {
             script_remove (weechat_python_plugin,
                            &python_scripts, &last_python_script,
                            python_current_script);
         }
-        
+
         Py_EndInterpreter (python_current_interpreter);
         /* PyEval_ReleaseLock (); */
-        
+
         return 0;
     }
 
@@ -589,14 +709,14 @@ weechat_python_load (const char *filename)
         PyErr_Print ();
 
     fclose (fp);
-    
+
     if (!python_registered_script)
     {
         weechat_printf (NULL,
                         weechat_gettext ("%s%s: function \"register\" not "
                                          "found (or failed) in file \"%s\""),
                         weechat_prefix ("error"), PYTHON_PLUGIN_NAME, filename);
-        
+
         if (PyErr_Occurred ())
             PyErr_Print ();
         Py_EndInterpreter (python_current_interpreter);
@@ -605,10 +725,10 @@ weechat_python_load (const char *filename)
         return 0;
     }
     python_current_script = python_registered_script;
-    
+
     python_current_script->interpreter = (PyThreadState *) python_current_interpreter;
     /* PyEval_ReleaseThread (python_current_script->interpreter); */
-    
+
     /*
      * set input/close callbacks for buffers created by this script
      * (to restore callbacks after upgrade)
@@ -618,7 +738,7 @@ weechat_python_load (const char *filename)
                                  python_current_script,
                                  &weechat_python_api_buffer_input_data_cb,
                                  &weechat_python_api_buffer_close_cb);
-    
+
     return 1;
 }
 
@@ -631,7 +751,7 @@ weechat_python_load_cb (void *data, const char *filename)
 {
     /* make C compiler happy */
     (void) data;
-    
+
     weechat_python_load (filename);
 }
 
@@ -642,38 +762,38 @@ weechat_python_load_cb (void *data, const char *filename)
 void
 weechat_python_unload (struct t_plugin_script *script)
 {
-    int *r;
+    int *rc;
     void *interpreter;
     PyThreadState *old_interpreter;
-    
+
     if ((weechat_python_plugin->debug >= 1) || !python_quiet)
     {
         weechat_printf (NULL,
                         weechat_gettext ("%s: unloading script \"%s\""),
                         PYTHON_PLUGIN_NAME, script->name);
     }
-    
+
     if (script->shutdown_func && script->shutdown_func[0])
     {
-        r = (int *) weechat_python_exec (script, WEECHAT_SCRIPT_EXEC_INT,
-                                         script->shutdown_func, NULL, NULL);
-        if (r)
-            free (r);
+        rc = (int *) weechat_python_exec (script, WEECHAT_SCRIPT_EXEC_INT,
+                                          script->shutdown_func, NULL, NULL);
+        if (rc)
+            free (rc);
     }
 
     old_interpreter = PyThreadState_Swap (NULL);
     interpreter = script->interpreter;
-    
+
     if (python_current_script == script)
         python_current_script = (python_current_script->prev_script) ?
             python_current_script->prev_script : python_current_script->next_script;
-    
+
     script_remove (weechat_python_plugin, &python_scripts, &last_python_script,
                    script);
-    
+
     PyThreadState_Swap (interpreter);
     Py_EndInterpreter (interpreter);
-    
+
     if (old_interpreter)
         PyThreadState_Swap (old_interpreter);
 }
@@ -686,7 +806,7 @@ void
 weechat_python_unload_name (const char *name)
 {
     struct t_plugin_script *ptr_script;
-    
+
     ptr_script = script_search (weechat_python_plugin, python_scripts, name);
     if (ptr_script)
     {
@@ -725,7 +845,7 @@ weechat_python_reload_name (const char *name)
 {
     struct t_plugin_script *ptr_script;
     char *filename;
-    
+
     ptr_script = script_search (weechat_python_plugin, python_scripts, name);
     if (ptr_script)
     {
@@ -757,7 +877,7 @@ weechat_python_command_cb (void *data, struct t_gui_buffer *buffer,
                            int argc, char **argv, char **argv_eol)
 {
     char *path_script;
-    
+
     /* make C compiler happy */
     (void) data;
     (void) buffer;
@@ -833,7 +953,7 @@ weechat_python_command_cb (void *data, struct t_gui_buffer *buffer,
                             "python");
         }
     }
-    
+
     return WEECHAT_RC_OK;
 }
 
@@ -850,9 +970,9 @@ weechat_python_completion_cb (void *data, const char *completion_item,
     (void) data;
     (void) completion_item;
     (void) buffer;
-    
+
     script_completion (weechat_python_plugin, completion, python_scripts);
-    
+
     return WEECHAT_RC_OK;
 }
 
@@ -867,12 +987,12 @@ weechat_python_info_cb (void *data, const char *info_name,
     /* make C compiler happy */
     (void) data;
     (void) arguments;
-    
+
     if (weechat_strcasecmp (info_name, "python2_bin") == 0)
     {
         return python2_bin;
     }
-    
+
     return NULL;
 }
 
@@ -886,17 +1006,17 @@ weechat_python_infolist_cb (void *data, const char *infolist_name,
 {
     /* make C compiler happy */
     (void) data;
-    
+
     if (!infolist_name || !infolist_name[0])
         return NULL;
-    
+
     if (weechat_strcasecmp (infolist_name, "python_script") == 0)
     {
         return script_infolist_list_scripts (weechat_python_plugin,
                                              python_scripts, pointer,
                                              arguments);
     }
-    
+
     return NULL;
 }
 
@@ -913,13 +1033,13 @@ weechat_python_signal_debug_dump_cb (void *data, const char *signal,
     (void) data;
     (void) signal;
     (void) type_data;
-    
+
     if (!signal_data
         || (weechat_strcasecmp ((char *)signal_data, PYTHON_PLUGIN_NAME) == 0))
     {
         script_print_log (weechat_python_plugin, python_scripts);
     }
-    
+
     return WEECHAT_RC_OK;
 }
 
@@ -937,10 +1057,10 @@ weechat_python_signal_buffer_closed_cb (void *data, const char *signal,
     (void) data;
     (void) signal;
     (void) type_data;
-    
+
     if (signal_data)
         script_remove_buffer_callbacks (python_scripts, signal_data);
-    
+
     return WEECHAT_RC_OK;
 }
 
@@ -972,7 +1092,7 @@ weechat_python_timer_action_cb (void *data, int remaining_calls)
                                   &python_action_remove_list);
         }
     }
-    
+
     return WEECHAT_RC_OK;
 }
 
@@ -988,7 +1108,7 @@ weechat_python_signal_script_action_cb (void *data, const char *signal,
 {
     /* make C compiler happy */
     (void) data;
-    
+
     if (strcmp (type_data, WEECHAT_HOOK_SIGNAL_STRING) == 0)
     {
         if (strcmp (signal, "python_script_install") == 0)
@@ -1008,7 +1128,7 @@ weechat_python_signal_script_action_cb (void *data, const char *signal,
                                 &python_action_remove_list);
         }
     }
-    
+
     return WEECHAT_RC_OK;
 }
 
@@ -1020,7 +1140,7 @@ int
 weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
 {
     weechat_python_plugin = plugin;
-    
+
     /*
      * hook info to get path to python 2.x interpreter
      * (some scripts using hook_process need that)
@@ -1030,10 +1150,13 @@ weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
                        N_("path to python 2.x interpreter"),
                        NULL,
                        &weechat_python_info_cb, NULL);
-    
+
     /* init stdout/stderr buffer */
     python_buffer_output[0] = '\0';
-    
+
+    PyImport_AppendInittab("weechat",
+                           &weechat_python_init_module_weechat);
+
     Py_Initialize ();
     if (Py_IsInitialized () == 0)
     {
@@ -1043,13 +1166,13 @@ weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
                         weechat_prefix ("error"), PYTHON_PLUGIN_NAME);
         return WEECHAT_RC_ERROR;
     }
-    
-    PyEval_InitThreads();
+
+    /* PyEval_InitThreads(); */
     /* python_mainThreadState = PyThreadState_Swap(NULL); */
     python_mainThreadState = PyEval_SaveThread();
     /* PyEval_ReleaseLock (); */
 
-    if (python_mainThreadState == NULL)
+    if (!python_mainThreadState)
     {
         weechat_printf (NULL,
                         weechat_gettext ("%s%s: unable to get current "
@@ -1070,10 +1193,10 @@ weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
                  &weechat_python_signal_script_action_cb,
                  &weechat_python_load_cb);
     python_quiet = 0;
-    
+
     script_display_short_list (weechat_python_plugin,
                                python_scripts);
-    
+
     /* init ok */
     return WEECHAT_RC_OK;
 }
@@ -1089,7 +1212,7 @@ weechat_plugin_end (struct t_weechat_plugin *plugin)
     python_quiet = 1;
     script_end (plugin, &python_scripts, &weechat_python_unload_all);
     python_quiet = 0;
-    
+
     /* free Python interpreter */
     if (python_mainThreadState != NULL)
     {
@@ -1098,7 +1221,7 @@ weechat_plugin_end (struct t_weechat_plugin *plugin)
         /* PyEval_ReleaseLock (); */
         python_mainThreadState = NULL;
     }
-    
+
     Py_Finalize ();
     if (Py_IsInitialized () != 0)
     {
@@ -1106,7 +1229,7 @@ weechat_plugin_end (struct t_weechat_plugin *plugin)
                         weechat_gettext ("%s%s: unable to free interpreter"),
                         weechat_prefix ("error"), PYTHON_PLUGIN_NAME);
     }
-    
+
     /* free some data */
     if (python2_bin)
         free (python2_bin);
@@ -1114,6 +1237,6 @@ weechat_plugin_end (struct t_weechat_plugin *plugin)
         free (python_action_install_list);
     if (python_action_remove_list)
         free (python_action_remove_list);
-    
+
     return WEECHAT_RC_OK;
 }
