@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2011 Sebastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2012 Sebastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -42,7 +42,7 @@ script_api_charset_set (struct t_plugin_script *script,
 {
     if (script->charset)
         free (script->charset);
-    
+
     script->charset = (charset) ? strdup (charset) : NULL;
 }
 
@@ -68,7 +68,7 @@ script_api_config_new (struct t_weechat_plugin *weechat_plugin,
         new_script_callback = script_callback_alloc ();
         if (!new_script_callback)
             return NULL;
-        
+
         new_config_file = weechat_config_new (name, callback_reload,
                                               new_script_callback);
         if (!new_config_file)
@@ -77,17 +77,17 @@ script_api_config_new (struct t_weechat_plugin *weechat_plugin,
             free (new_script_callback);
             return NULL;
         }
-        
+
         script_callback_init (new_script_callback, script, function, data);
         new_script_callback->config_file = new_config_file;
-        
+
         script_callback_add (script, new_script_callback);
     }
     else
     {
         new_config_file = weechat_config_new (name, NULL, NULL);
     }
-    
+
     return new_config_file;
 }
 
@@ -139,7 +139,7 @@ script_api_config_new_section (struct t_weechat_plugin *weechat_plugin,
     struct t_script_callback *new_script_callback5;
     struct t_config_section *new_section;
     void *callback1, *callback2, *callback3, *callback4, *callback5;
-    
+
     new_script_callback1 = NULL;
     new_script_callback2 = NULL;
     new_script_callback3 = NULL;
@@ -150,7 +150,7 @@ script_api_config_new_section (struct t_weechat_plugin *weechat_plugin,
     callback3 = NULL;
     callback4 = NULL;
     callback5 = NULL;
-    
+
     if (function_read && function_read[0])
     {
         new_script_callback1 = script_callback_alloc ();
@@ -158,7 +158,7 @@ script_api_config_new_section (struct t_weechat_plugin *weechat_plugin,
             return NULL;
         callback1 = callback_read;
     }
-    
+
     if (function_write && function_write[0])
     {
         new_script_callback2 = script_callback_alloc ();
@@ -193,7 +193,7 @@ script_api_config_new_section (struct t_weechat_plugin *weechat_plugin,
         }
         callback3 = callback_write_default;
     }
-    
+
     if (function_create_option && function_create_option[0])
     {
         new_script_callback4 = script_callback_alloc ();
@@ -218,7 +218,7 @@ script_api_config_new_section (struct t_weechat_plugin *weechat_plugin,
         }
         callback4 = callback_create_option;
     }
-    
+
     if (function_delete_option && function_delete_option[0])
     {
         new_script_callback5 = script_callback_alloc ();
@@ -248,7 +248,7 @@ script_api_config_new_section (struct t_weechat_plugin *weechat_plugin,
         }
         callback5 = callback_delete_option;
     }
-    
+
     new_section = weechat_config_new_section (config_file,
                                               name,
                                               user_can_add_options,
@@ -310,7 +310,7 @@ script_api_config_new_section (struct t_weechat_plugin *weechat_plugin,
         new_script_callback2->config_section = new_section;
         script_callback_add (script, new_script_callback2);
     }
-    
+
     if (new_script_callback3)
     {
         script_callback_init (new_script_callback3, script,
@@ -319,7 +319,7 @@ script_api_config_new_section (struct t_weechat_plugin *weechat_plugin,
         new_script_callback3->config_section = new_section;
         script_callback_add (script, new_script_callback3);
     }
-    
+
     if (new_script_callback4)
     {
         script_callback_init (new_script_callback4, script,
@@ -328,7 +328,7 @@ script_api_config_new_section (struct t_weechat_plugin *weechat_plugin,
         new_script_callback4->config_section = new_section;
         script_callback_add (script, new_script_callback4);
     }
-    
+
     if (new_script_callback5)
     {
         script_callback_init (new_script_callback5, script,
@@ -337,7 +337,7 @@ script_api_config_new_section (struct t_weechat_plugin *weechat_plugin,
         new_script_callback5->config_section = new_section;
         script_callback_add (script, new_script_callback5);
     }
-    
+
     return new_section;
 }
 
@@ -375,14 +375,14 @@ script_api_config_new_option (struct t_weechat_plugin *weechat_plugin,
     struct t_script_callback *new_script_callback3;
     void *callback1, *callback2, *callback3;
     struct t_config_option *new_option;
-    
+
     new_script_callback1 = NULL;
     new_script_callback2 = NULL;
     new_script_callback3 = NULL;
     callback1 = NULL;
     callback2 = NULL;
     callback3 = NULL;
-    
+
     if (function_check_value && function_check_value[0])
     {
         new_script_callback1 = script_callback_alloc ();
@@ -390,7 +390,7 @@ script_api_config_new_option (struct t_weechat_plugin *weechat_plugin,
             return NULL;
         callback1 = callback_check_value;
     }
-    
+
     if (function_change && function_change[0])
     {
         new_script_callback2 = script_callback_alloc ();
@@ -425,7 +425,7 @@ script_api_config_new_option (struct t_weechat_plugin *weechat_plugin,
         }
         callback3 = callback_delete;
     }
-    
+
     new_option = weechat_config_new_option (config_file, section, name, type,
                                             description, string_values, min,
                                             max, default_value, value,
@@ -452,7 +452,7 @@ script_api_config_new_option (struct t_weechat_plugin *weechat_plugin,
         }
         return NULL;
     }
-    
+
     if (new_script_callback1)
     {
         script_callback_init (new_script_callback1, script,
@@ -462,7 +462,7 @@ script_api_config_new_option (struct t_weechat_plugin *weechat_plugin,
         new_script_callback1->config_option = new_option;
         script_callback_add (script, new_script_callback1);
     }
-    
+
     if (new_script_callback2)
     {
         script_callback_init (new_script_callback2, script,
@@ -472,7 +472,7 @@ script_api_config_new_option (struct t_weechat_plugin *weechat_plugin,
         new_script_callback2->config_option = new_option;
         script_callback_add (script, new_script_callback2);
     }
-    
+
     if (new_script_callback3)
     {
         script_callback_init (new_script_callback3, script,
@@ -482,7 +482,7 @@ script_api_config_new_option (struct t_weechat_plugin *weechat_plugin,
         new_script_callback3->config_option = new_option;
         script_callback_add (script, new_script_callback3);
     }
-    
+
     return new_option;
 }
 
@@ -496,20 +496,20 @@ script_api_config_option_free (struct t_weechat_plugin *weechat_plugin,
                                struct t_config_option *option)
 {
     struct t_script_callback *ptr_script_callback, *next_callback;
-    
+
     if (!weechat_plugin || !script || !option)
         return;
-    
+
     weechat_config_option_free (option);
-    
+
     ptr_script_callback = script->callbacks;
     while (ptr_script_callback)
     {
         next_callback = ptr_script_callback->next_callback;
-        
+
         if (ptr_script_callback->config_option == option)
             script_callback_remove (script, ptr_script_callback);
-        
+
         ptr_script_callback = next_callback;
     }
 }
@@ -525,21 +525,21 @@ script_api_config_section_free_options (struct t_weechat_plugin *weechat_plugin,
                                         struct t_config_section *section)
 {
     struct t_script_callback *ptr_script_callback, *next_callback;
-    
+
     if (!weechat_plugin || !script || !section)
         return;
-    
+
     weechat_config_section_free_options (section);
-    
+
     ptr_script_callback = script->callbacks;
     while (ptr_script_callback)
     {
         next_callback = ptr_script_callback->next_callback;
-        
+
         if ((ptr_script_callback->config_section == section)
             && ptr_script_callback->config_option)
             script_callback_remove (script, ptr_script_callback);
-        
+
         ptr_script_callback = next_callback;
     }
 }
@@ -554,20 +554,20 @@ script_api_config_section_free (struct t_weechat_plugin *weechat_plugin,
                                 struct t_config_section *section)
 {
     struct t_script_callback *ptr_script_callback, *next_callback;
-    
+
     if (!weechat_plugin || !script || !section)
         return;
-    
+
     weechat_config_section_free (section);
-    
+
     ptr_script_callback = script->callbacks;
     while (ptr_script_callback)
     {
         next_callback = ptr_script_callback->next_callback;
-        
+
         if (ptr_script_callback->config_section == section)
             script_callback_remove (script, ptr_script_callback);
-        
+
         ptr_script_callback = next_callback;
     }
 }
@@ -582,20 +582,20 @@ script_api_config_free (struct t_weechat_plugin *weechat_plugin,
                         struct t_config_file *config_file)
 {
     struct t_script_callback *ptr_script_callback, *next_callback;
-    
+
     if (!weechat_plugin || !script || !config_file)
         return;
-    
+
     weechat_config_free (config_file);
 
     ptr_script_callback = script->callbacks;
     while (ptr_script_callback)
     {
         next_callback = ptr_script_callback->next_callback;
-        
+
         if (ptr_script_callback->config_file == config_file)
             script_callback_remove (script, ptr_script_callback);
-        
+
         ptr_script_callback = next_callback;
     }
 }
@@ -610,17 +610,17 @@ script_api_printf (struct t_weechat_plugin *weechat_plugin,
                    struct t_gui_buffer *buffer, const char *format, ...)
 {
     char *buf2;
-    
+
     weechat_va_format (format);
     if (!vbuffer)
         return;
-    
+
     buf2 = (script && script->charset && script->charset[0]) ?
         weechat_iconv_to_internal (script->charset, vbuffer) : NULL;
     weechat_printf (buffer, "%s", (buf2) ? buf2 : vbuffer);
     if (buf2)
         free (buf2);
-    
+
     free (vbuffer);
 }
 
@@ -636,18 +636,18 @@ script_api_printf_date_tags (struct t_weechat_plugin *weechat_plugin,
                              const char *format, ...)
 {
     char *buf2;
-    
+
     weechat_va_format (format);
     if (!vbuffer)
         return;
-    
+
     buf2 = (script->charset && script->charset[0]) ?
         weechat_iconv_to_internal (script->charset, vbuffer) : NULL;
     weechat_printf_date_tags (buffer, date, tags,
                               "%s", (buf2) ? buf2 : vbuffer);
     if (buf2)
         free (buf2);
-    
+
     free (vbuffer);
 }
 
@@ -662,17 +662,17 @@ script_api_printf_y (struct t_weechat_plugin *weechat_plugin,
                      const char *format, ...)
 {
     char *buf2;
-    
+
     weechat_va_format (format);
     if (!vbuffer)
         return;
-    
+
     buf2 = (script->charset && script->charset[0]) ?
         weechat_iconv_to_internal (script->charset, vbuffer) : NULL;
     weechat_printf_y (buffer, y, "%s", (buf2) ? buf2 : vbuffer);
     if (buf2)
         free (buf2);
-    
+
     free (vbuffer);
 }
 
@@ -686,17 +686,17 @@ script_api_log_printf (struct t_weechat_plugin *weechat_plugin,
                        const char *format, ...)
 {
     char *buf2;
-    
+
     weechat_va_format (format);
     if (!vbuffer)
         return;
-    
+
     buf2 = (script->charset && script->charset[0]) ?
         weechat_iconv_to_internal (script->charset, vbuffer) : NULL;
     weechat_log_printf ("%s", (buf2) ? buf2 : vbuffer);
     if (buf2)
         free (buf2);
-    
+
     free (vbuffer);
 }
 
@@ -724,7 +724,7 @@ script_api_hook_command (struct t_weechat_plugin *weechat_plugin,
     new_script_callback = script_callback_alloc ();
     if (!new_script_callback)
         return NULL;
-    
+
     new_hook = weechat_hook_command (command, description, args,
                                      args_description, completion,
                                      callback, new_script_callback);
@@ -737,9 +737,9 @@ script_api_hook_command (struct t_weechat_plugin *weechat_plugin,
 
     script_callback_init (new_script_callback, script, function, data);
     new_script_callback->hook = new_hook;
-    
+
     script_callback_add (script, new_script_callback);
-    
+
     return new_hook;
 }
 
@@ -760,11 +760,11 @@ script_api_hook_command_run (struct t_weechat_plugin *weechat_plugin,
 {
     struct t_script_callback *new_script_callback;
     struct t_hook *new_hook;
-    
+
     new_script_callback = script_callback_alloc ();
     if (!new_script_callback)
         return NULL;
-    
+
     new_hook = weechat_hook_command_run (command,
                                          callback, new_script_callback);
     if (!new_hook)
@@ -773,12 +773,12 @@ script_api_hook_command_run (struct t_weechat_plugin *weechat_plugin,
         free (new_script_callback);
         return NULL;
     }
-    
+
     script_callback_init (new_script_callback, script, function, data);
     new_script_callback->hook = new_hook;
-    
+
     script_callback_add (script, new_script_callback);
-    
+
     return new_hook;
 }
 
@@ -798,11 +798,11 @@ script_api_hook_timer (struct t_weechat_plugin *weechat_plugin,
 {
     struct t_script_callback *new_script_callback;
     struct t_hook *new_hook;
-    
+
     new_script_callback = script_callback_alloc ();
     if (!new_script_callback)
         return NULL;
-    
+
     new_hook = weechat_hook_timer (interval, align_second, max_calls,
                                    callback, new_script_callback);
     if (!new_hook)
@@ -811,12 +811,12 @@ script_api_hook_timer (struct t_weechat_plugin *weechat_plugin,
         free (new_script_callback);
         return NULL;
     }
-    
+
     script_callback_init (new_script_callback, script, function, data);
     new_script_callback->hook = new_hook;
-    
+
     script_callback_add (script, new_script_callback);
-    
+
     return new_hook;
 }
 
@@ -836,11 +836,11 @@ script_api_hook_fd (struct t_weechat_plugin *weechat_plugin,
 {
     struct t_script_callback *new_script_callback;
     struct t_hook *new_hook;
-    
+
     new_script_callback = script_callback_alloc ();
     if (!new_script_callback)
         return NULL;
-    
+
     new_hook = weechat_hook_fd (fd, flag_read, flag_write, flag_exception,
                                 callback, new_script_callback);
     if (!new_hook)
@@ -849,17 +849,60 @@ script_api_hook_fd (struct t_weechat_plugin *weechat_plugin,
         free (new_script_callback);
         return NULL;
     }
-    
+
     script_callback_init (new_script_callback, script, function, data);
     new_script_callback->hook = new_hook;
-    
+
     script_callback_add (script, new_script_callback);
-    
+
     return new_hook;
 }
 
 /*
- * script_api_hook_connect: hook a connection
+ * script_api_hook_process_hashtable: hook a process
+ *                                    return new hook, NULL if error
+ */
+
+struct t_hook *
+script_api_hook_process_hashtable (struct t_weechat_plugin *weechat_plugin,
+                                   struct t_plugin_script *script,
+                                   const char *command,
+                                   struct t_hashtable *options,
+                                   int timeout,
+                                   int (*callback)(void *data,
+                                                   const char *command,
+                                                   int return_code,
+                                                   const char *out,
+                                                   const char *err),
+                                   const char *function,
+                                   const char *data)
+{
+    struct t_script_callback *new_script_callback;
+    struct t_hook *new_hook;
+
+    new_script_callback = script_callback_alloc ();
+    if (!new_script_callback)
+        return NULL;
+
+    script_callback_init (new_script_callback, script, function, data);
+    script_callback_add (script, new_script_callback);
+
+    new_hook = weechat_hook_process_hashtable (command, options, timeout,
+                                               callback, new_script_callback);
+
+    if (!new_hook)
+    {
+        script_callback_remove (script, new_script_callback);
+        return NULL;
+    }
+
+    new_script_callback->hook = new_hook;
+
+    return new_hook;
+}
+
+/*
+ * script_api_hook_process: hook a process
  *                          return new hook, NULL if error
  */
 
@@ -876,28 +919,9 @@ script_api_hook_process (struct t_weechat_plugin *weechat_plugin,
                          const char *function,
                          const char *data)
 {
-    struct t_script_callback *new_script_callback;
-    struct t_hook *new_hook;
-    
-    new_script_callback = script_callback_alloc ();
-    if (!new_script_callback)
-        return NULL;
-    
-    script_callback_init (new_script_callback, script, function, data);
-    script_callback_add (script, new_script_callback);
-    
-    new_hook = weechat_hook_process (command, timeout, callback,
-                                     new_script_callback);
-    
-    if (!new_hook)
-    {
-        script_callback_remove (script, new_script_callback);
-        return NULL;
-    }
-    
-    new_script_callback->hook = new_hook;
-    
-    return new_hook;
+    return script_api_hook_process_hashtable (weechat_plugin, script, command,
+                                              NULL, timeout,
+                                              callback, function, data);
 }
 
 /*
@@ -926,7 +950,7 @@ script_api_hook_connect (struct t_weechat_plugin *weechat_plugin,
     new_script_callback = script_callback_alloc ();
     if (!new_script_callback)
         return NULL;
-    
+
     new_hook = weechat_hook_connect (proxy, address, port, sock, ipv6,
                                      gnutls_sess, gnutls_cb, gnutls_dhkey_size,
                                      gnutls_priorities, local_hostname,
@@ -937,12 +961,12 @@ script_api_hook_connect (struct t_weechat_plugin *weechat_plugin,
         free (new_script_callback);
         return NULL;
     }
-    
+
     script_callback_init (new_script_callback, script, function, data);
     new_script_callback->hook = new_hook;
-    
+
     script_callback_add (script, new_script_callback);
-    
+
     return new_hook;
 }
 
@@ -968,11 +992,11 @@ script_api_hook_print (struct t_weechat_plugin *weechat_plugin,
 {
     struct t_script_callback *new_script_callback;
     struct t_hook *new_hook;
-    
+
     new_script_callback = script_callback_alloc ();
     if (!new_script_callback)
         return NULL;
-    
+
     new_hook = weechat_hook_print (buffer, tags, message, strip_colors,
                                    callback, new_script_callback);
     if (!new_hook)
@@ -981,12 +1005,12 @@ script_api_hook_print (struct t_weechat_plugin *weechat_plugin,
         free (new_script_callback);
         return NULL;
     }
-    
+
     script_callback_init (new_script_callback, script, function, data);
     new_script_callback->hook = new_hook;
-    
+
     script_callback_add (script, new_script_callback);
-    
+
     return new_hook;
 }
 
@@ -1007,11 +1031,11 @@ script_api_hook_signal (struct t_weechat_plugin *weechat_plugin,
 {
     struct t_script_callback *new_script_callback;
     struct t_hook *new_hook;
-    
+
     new_script_callback = script_callback_alloc ();
     if (!new_script_callback)
         return NULL;
-    
+
     new_hook = weechat_hook_signal (signal, callback, new_script_callback);
     if (!new_hook)
     {
@@ -1019,12 +1043,12 @@ script_api_hook_signal (struct t_weechat_plugin *weechat_plugin,
         free (new_script_callback);
         return NULL;
     }
-    
+
     script_callback_init (new_script_callback, script, function, data);
     new_script_callback->hook = new_hook;
-    
+
     script_callback_add (script, new_script_callback);
-    
+
     return new_hook;
 }
 
@@ -1044,11 +1068,11 @@ script_api_hook_hsignal (struct t_weechat_plugin *weechat_plugin,
 {
     struct t_script_callback *new_script_callback;
     struct t_hook *new_hook;
-    
+
     new_script_callback = script_callback_alloc ();
     if (!new_script_callback)
         return NULL;
-    
+
     new_hook = weechat_hook_hsignal (signal, callback, new_script_callback);
     if (!new_hook)
     {
@@ -1056,12 +1080,12 @@ script_api_hook_hsignal (struct t_weechat_plugin *weechat_plugin,
         free (new_script_callback);
         return NULL;
     }
-    
+
     script_callback_init (new_script_callback, script, function, data);
     new_script_callback->hook = new_hook;
-    
+
     script_callback_add (script, new_script_callback);
-    
+
     return new_hook;
 }
 
@@ -1081,11 +1105,11 @@ script_api_hook_config (struct t_weechat_plugin *weechat_plugin,
 {
     struct t_script_callback *new_script_callback;
     struct t_hook *new_hook;
-    
+
     new_script_callback = script_callback_alloc ();
     if (!new_script_callback)
         return NULL;
-    
+
     new_hook = weechat_hook_config (option, callback, new_script_callback);
     if (!new_hook)
     {
@@ -1093,12 +1117,12 @@ script_api_hook_config (struct t_weechat_plugin *weechat_plugin,
         free (new_script_callback);
         return NULL;
     }
-    
+
     script_callback_init (new_script_callback, script, function, data);
     new_script_callback->hook = new_hook;
-    
+
     script_callback_add (script, new_script_callback);
-    
+
     return new_hook;
 }
 
@@ -1121,11 +1145,11 @@ script_api_hook_completion (struct t_weechat_plugin *weechat_plugin,
 {
     struct t_script_callback *new_script_callback;
     struct t_hook *new_hook;
-    
+
     new_script_callback = script_callback_alloc ();
     if (!new_script_callback)
         return NULL;
-    
+
     new_hook = weechat_hook_completion (completion, description,
                                         callback, new_script_callback);
     if (!new_hook)
@@ -1134,12 +1158,12 @@ script_api_hook_completion (struct t_weechat_plugin *weechat_plugin,
         free (new_script_callback);
         return NULL;
     }
-    
+
     script_callback_init (new_script_callback, script, function, data);
     new_script_callback->hook = new_hook;
-    
+
     script_callback_add (script, new_script_callback);
-    
+
     return new_hook;
 }
 
@@ -1160,11 +1184,11 @@ script_api_hook_modifier (struct t_weechat_plugin *weechat_plugin,
 {
     struct t_script_callback *new_script_callback;
     struct t_hook *new_hook;
-    
+
     new_script_callback = script_callback_alloc ();
     if (!new_script_callback)
         return NULL;
-    
+
     new_hook = weechat_hook_modifier (modifier, callback, new_script_callback);
     if (!new_hook)
     {
@@ -1172,12 +1196,12 @@ script_api_hook_modifier (struct t_weechat_plugin *weechat_plugin,
         free (new_script_callback);
         return NULL;
     }
-    
+
     script_callback_init (new_script_callback, script, function, data);
     new_script_callback->hook = new_hook;
-    
+
     script_callback_add (script, new_script_callback);
-    
+
     return new_hook;
 }
 
@@ -1200,11 +1224,11 @@ script_api_hook_info (struct t_weechat_plugin *weechat_plugin,
 {
     struct t_script_callback *new_script_callback;
     struct t_hook *new_hook;
-    
+
     new_script_callback = script_callback_alloc ();
     if (!new_script_callback)
         return NULL;
-    
+
     new_hook = weechat_hook_info (info_name, description, args_description,
                                   callback, new_script_callback);
     if (!new_hook)
@@ -1213,12 +1237,12 @@ script_api_hook_info (struct t_weechat_plugin *weechat_plugin,
         free (new_script_callback);
         return NULL;
     }
-    
+
     script_callback_init (new_script_callback, script, function, data);
     new_script_callback->hook = new_hook;
-    
+
     script_callback_add (script, new_script_callback);
-    
+
     return new_hook;
 }
 
@@ -1242,11 +1266,11 @@ script_api_hook_info_hashtable (struct t_weechat_plugin *weechat_plugin,
 {
     struct t_script_callback *new_script_callback;
     struct t_hook *new_hook;
-    
+
     new_script_callback = script_callback_alloc ();
     if (!new_script_callback)
         return NULL;
-    
+
     new_hook = weechat_hook_info_hashtable (info_name, description,
                                             args_description,
                                             output_description,
@@ -1257,12 +1281,12 @@ script_api_hook_info_hashtable (struct t_weechat_plugin *weechat_plugin,
         free (new_script_callback);
         return NULL;
     }
-    
+
     script_callback_init (new_script_callback, script, function, data);
     new_script_callback->hook = new_hook;
-    
+
     script_callback_add (script, new_script_callback);
-    
+
     return new_hook;
 }
 
@@ -1287,11 +1311,11 @@ script_api_hook_infolist (struct t_weechat_plugin *weechat_plugin,
 {
     struct t_script_callback *new_script_callback;
     struct t_hook *new_hook;
-    
+
     new_script_callback = script_callback_alloc ();
     if (!new_script_callback)
         return NULL;
-    
+
     new_hook = weechat_hook_infolist (infolist_name, description,
                                       pointer_description, args_description,
                                       callback, new_script_callback);
@@ -1301,12 +1325,12 @@ script_api_hook_infolist (struct t_weechat_plugin *weechat_plugin,
         free (new_script_callback);
         return NULL;
     }
-    
+
     script_callback_init (new_script_callback, script, function, data);
     new_script_callback->hook = new_hook;
-    
+
     script_callback_add (script, new_script_callback);
-    
+
     return new_hook;
 }
 
@@ -1326,11 +1350,11 @@ script_api_hook_focus (struct t_weechat_plugin *weechat_plugin,
 {
     struct t_script_callback *new_script_callback;
     struct t_hook *new_hook;
-    
+
     new_script_callback = script_callback_alloc ();
     if (!new_script_callback)
         return NULL;
-    
+
     new_hook = weechat_hook_focus (area, callback, new_script_callback);
     if (!new_hook)
     {
@@ -1338,12 +1362,12 @@ script_api_hook_focus (struct t_weechat_plugin *weechat_plugin,
         free (new_script_callback);
         return NULL;
     }
-    
+
     script_callback_init (new_script_callback, script, function, data);
     new_script_callback->hook = new_hook;
-    
+
     script_callback_add (script, new_script_callback);
-    
+
     return new_hook;
 }
 
@@ -1357,20 +1381,20 @@ script_api_unhook (struct t_weechat_plugin *weechat_plugin,
                    struct t_hook *hook)
 {
     struct t_script_callback *ptr_script_callback, *next_callback;
-    
+
     if (!weechat_plugin || !script || !hook)
         return;
-    
+
     weechat_unhook (hook);
-    
+
     ptr_script_callback = script->callbacks;
     while (ptr_script_callback)
     {
         next_callback = ptr_script_callback->next_callback;
-        
+
         if (ptr_script_callback->hook == hook)
             script_callback_remove (script, ptr_script_callback);
-        
+
         ptr_script_callback = next_callback;
     }
 }
@@ -1383,14 +1407,14 @@ void
 script_api_unhook_all (struct t_plugin_script *script)
 {
     struct t_script_callback *ptr_callback, *next_callback;
-    
+
     ptr_callback = script->callbacks;
     while (ptr_callback)
     {
         next_callback = ptr_callback->next_callback;
-        
+
         script_callback_remove (script, ptr_callback);
-        
+
         ptr_callback = next_callback;
     }
 }
@@ -1416,21 +1440,21 @@ script_api_buffer_new (struct t_weechat_plugin *weechat_plugin,
     struct t_script_callback *new_script_callback_input;
     struct t_script_callback *new_script_callback_close;
     struct t_gui_buffer *new_buffer;
-    
+
     if ((!function_input || !function_input[0])
         && (!function_close || !function_close[0]))
         return weechat_buffer_new (name, NULL, NULL, NULL, NULL);
-    
+
     new_script_callback_input = NULL;
     new_script_callback_close = NULL;
-    
+
     if (function_input && function_input[0])
     {
         new_script_callback_input = script_callback_alloc ();
         if (!new_script_callback_input)
             return NULL;
     }
-    
+
     if (function_close && function_close[0])
     {
         new_script_callback_close = script_callback_alloc ();
@@ -1444,7 +1468,7 @@ script_api_buffer_new (struct t_weechat_plugin *weechat_plugin,
             return NULL;
         }
     }
-    
+
     new_buffer = weechat_buffer_new (name,
                                      (new_script_callback_input) ?
                                      input_callback : NULL,
@@ -1468,7 +1492,7 @@ script_api_buffer_new (struct t_weechat_plugin *weechat_plugin,
         }
         return NULL;
     }
-    
+
     if (new_script_callback_input)
     {
         script_callback_init (new_script_callback_input,
@@ -1476,7 +1500,7 @@ script_api_buffer_new (struct t_weechat_plugin *weechat_plugin,
         new_script_callback_input->buffer = new_buffer;
         script_callback_add (script, new_script_callback_input);
     }
-    
+
     if (new_script_callback_close)
     {
         script_callback_init (new_script_callback_close,
@@ -1496,7 +1520,7 @@ script_api_buffer_new (struct t_weechat_plugin *weechat_plugin,
                         function_close);
     weechat_buffer_set (new_buffer, "localvar_set_script_close_cb_data",
                         data_close);
-    
+
     return new_buffer;
 }
 
@@ -1510,20 +1534,20 @@ script_api_buffer_close (struct t_weechat_plugin *weechat_plugin,
                          struct t_gui_buffer *buffer)
 {
     struct t_script_callback *ptr_script_callback, *next_callback;
-    
+
     if (!weechat_plugin || !script || !buffer)
         return;
-    
+
     weechat_buffer_close (buffer);
-    
+
     ptr_script_callback = script->callbacks;
     while (ptr_script_callback)
     {
         next_callback = ptr_script_callback->next_callback;
-        
+
         if (ptr_script_callback->buffer == buffer)
             script_callback_remove (script, ptr_script_callback);
-        
+
         ptr_script_callback = next_callback;
     }
 }
@@ -1544,13 +1568,13 @@ script_api_bar_item_new (struct t_weechat_plugin *weechat_plugin,
 {
     struct t_script_callback *new_script_callback;
     struct t_gui_bar_item *new_item;
-    
+
     new_script_callback = script_callback_alloc ();
     if (!new_script_callback)
         return NULL;
-    
+
     script_callback_init (new_script_callback, script, function, data);
-    
+
     new_item = weechat_bar_item_new (name,
                                      (function && function[0]) ?
                                      build_callback : NULL,
@@ -1562,10 +1586,10 @@ script_api_bar_item_new (struct t_weechat_plugin *weechat_plugin,
         free (new_script_callback);
         return NULL;
     }
-    
+
     new_script_callback->bar_item = new_item;
     script_callback_add (script, new_script_callback);
-    
+
     return new_item;
 }
 
@@ -1579,20 +1603,20 @@ script_api_bar_item_remove (struct t_weechat_plugin *weechat_plugin,
                             struct t_gui_bar_item *item)
 {
     struct t_script_callback *ptr_script_callback, *next_callback;
-    
+
     if (!weechat_plugin || !script || !item)
         return;
-    
+
     weechat_bar_item_remove (item);
 
     ptr_script_callback = script->callbacks;
     while (ptr_script_callback)
     {
         next_callback = ptr_script_callback->next_callback;
-        
+
         if (ptr_script_callback->bar_item == item)
             script_callback_remove (script, ptr_script_callback);
-        
+
         ptr_script_callback = next_callback;
     }
 }
@@ -1607,12 +1631,12 @@ script_api_command (struct t_weechat_plugin *weechat_plugin,
                     struct t_gui_buffer *buffer, const char *command)
 {
     char *command2;
-    
+
     command2 = (script->charset && script->charset[0]) ?
         weechat_iconv_to_internal (script->charset, command) : NULL;
-    
+
     weechat_command (buffer, (command2) ? command2 : command);
-    
+
     if (command2)
         free (command2);
 }
@@ -1629,19 +1653,19 @@ script_api_config_get_plugin (struct t_weechat_plugin *weechat_plugin,
 {
     char *option_fullname;
     const char *return_value;
-    
+
     option_fullname = malloc ((strlen (script->name) +
                                strlen (option) + 2));
     if (!option_fullname)
         return NULL;
-    
+
     strcpy (option_fullname, script->name);
     strcat (option_fullname, ".");
     strcat (option_fullname, option);
-    
+
     return_value = weechat_config_get_plugin (option_fullname);
     free (option_fullname);
-    
+
     return return_value;
 }
 
@@ -1656,19 +1680,19 @@ script_api_config_is_set_plugin (struct t_weechat_plugin *weechat_plugin,
 {
     char *option_fullname;
     int return_code;
-    
+
     option_fullname = malloc ((strlen (script->name) +
                                strlen (option) + 2));
     if (!option_fullname)
         return 0;
-    
+
     strcpy (option_fullname, script->name);
     strcat (option_fullname, ".");
     strcat (option_fullname, option);
-    
+
     return_code = weechat_config_is_set_plugin (option_fullname);
     free (option_fullname);
-    
+
     return return_code;
 }
 
@@ -1684,19 +1708,19 @@ script_api_config_set_plugin (struct t_weechat_plugin *weechat_plugin,
 {
     char *option_fullname;
     int return_code;
-    
+
     option_fullname = malloc ((strlen (script->name) +
                                strlen (option) + 2));
     if (!option_fullname)
         return 0;
-    
+
     strcpy (option_fullname, script->name);
     strcat (option_fullname, ".");
     strcat (option_fullname, option);
-    
+
     return_code = weechat_config_set_plugin (option_fullname, value);
     free (option_fullname);
-    
+
     return return_code;
 }
 
@@ -1711,16 +1735,16 @@ script_api_config_set_desc_plugin (struct t_weechat_plugin *weechat_plugin,
                                    const char *option, const char *description)
 {
     char *option_fullname;
-    
+
     option_fullname = malloc ((strlen (script->name) +
                                strlen (option) + 2));
     if (!option_fullname)
         return;
-    
+
     strcpy (option_fullname, script->name);
     strcat (option_fullname, ".");
     strcat (option_fullname, option);
-    
+
     weechat_config_set_desc_plugin (option_fullname, description);
     free (option_fullname);
 }
@@ -1737,19 +1761,19 @@ script_api_config_unset_plugin (struct t_weechat_plugin *weechat_plugin,
 {
     char *option_fullname;
     int return_code;
-    
+
     option_fullname = malloc ((strlen (script->name) +
                                strlen (option) + 2));
     if (!option_fullname)
         return 0;
-    
+
     strcpy (option_fullname, script->name);
     strcat (option_fullname, ".");
     strcat (option_fullname, option);
-    
+
     return_code = weechat_config_unset_plugin (option_fullname);
     free (option_fullname);
-    
+
     return return_code;
 }
 
@@ -1771,23 +1795,23 @@ script_api_upgrade_read (struct t_weechat_plugin *weechat_plugin,
 {
     struct t_script_callback *new_script_callback;
     int rc;
-    
+
     if (!function || !function[0])
         return 0;
-    
+
     new_script_callback = script_callback_alloc ();
     if (!new_script_callback)
         return 0;
-    
+
     script_callback_init (new_script_callback, script, function, data);
     new_script_callback->upgrade_file = upgrade_file;
     script_callback_add (script, new_script_callback);
-    
+
     rc = weechat_upgrade_read (upgrade_file,
                                callback_read,
                                new_script_callback);
-    
+
     script_callback_remove (script, new_script_callback);
-    
+
     return rc;
 }
